program DDEnum_api;
{******************************************************************}
{                                                                  }
{       Borland Delphi DirectX 8 SDK Examples                      }
{       Conversion of the Direct Draw Enumeration Example          }
{                                                                  }
{ Portions created by Microsoft are                                }
{ Copyright (C) 1995-2000 Microsoft Corporation.                   }
{ All Rights Reserved.                                             }
{                                                                  }
{ The original files are : DDEnum.h, DDEnum.cpp                    }
{ The original Pascal code is : DDEnum_api.pas                     }
{ The initial developer of the Pascal code is : Dominique Louis    }
{ ( Dominique@SavageSoftware.com.au )                              }
{                                                                  }
{ Portions created by Dominique Louis are                          }
{ Copyright (C) 2000 - 2001 Dominique Louis.                       }
{                                                                  }
{ Contributor(s)                                                   }
{ --------------                                                   }
{ <Contributer Name> ( contributer@sp.sw )                         }
{                                                                  }
{ Obtained through:                                                }
{ Joint Endeavour of Delphi Innovators ( Project JEDI )            }
{                                                                  }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{                                                                  }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/NPL/NPL-1_1Final.html                     }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{ Description                                                      }
{ -----------                                                      }
{   This sample shows how to enumerate the current DirectDraw      }
{   devices and how to get information using the                   }
{   IDirectDraw4::GetDeviceIdentifier method.                      }
{                                                                  }
{ Requires                                                         }
{ --------                                                         }
{   Eric Unger and Tim Baumgarten's conversions of the DirectX     }
{   headers. They are available from...                            }
{   http://delphi-jedi.org/DelphiGraphics/ .                       }
{                                                                  }
{                                                                  }
{ Programming Notes                                                }
{ -----------------                                                }
{                                                                  }
{                                                                  }
{ Revision History                                                 }
{ ----------------                                                 }
{   2000-12-01 - DL  Initial translation.                          }
{                                                                  }
{                                                                  }
{******************************************************************}
uses
  Windows,
  SysUtils,
  Messages,
  DirectDraw;

{$R *.RES}
const
  NAME : PChar = 'DDEnum';
  TITLE : PChar = 'DirectDraw Enumeration Sample';
  MAX_DEVICES = 32;
  IDI_MAIN_ICON = 101;
  IDD_DRIVERINFO = 102;
  IDC_PREV = 1001;
  IDC_NEXT = 1002;
  IDC_DEVICEID = 1003;
  IDC_VERSION = 1004;
  IDC_DESCRIPTION = 1005;
  IDC_FILENAME = 1006;
  IDC_COUNT = 1007;
  IDC_GUID = 1010;
  IDC_DWVENDORID = 1012;
  IDC_DWDEVICEID = 1014;
  IDC_DWSUBSYS = 1015;
  IDC_DWREVISION = 1016;
  IDC_RADIO_DEVICE = 1018;
  IDC_RADIO_HOST = 1019;
  IDC_STATIC_WHQLLEVEL = 1020;

type
  TDeviceInfo = packed record
    DeviceInfo : TDDDeviceIdentifier2;
    DeviceInfoHost : TDDDeviceIdentifier2;
  end;

var
  //---------------------------------------------------------------------------
  // Global data
  //---------------------------------------------------------------------------
  g_DeviceIdent : array[ 0..MAX_DEVICES - 1 ] of TDeviceInfo;
  g_iMaxDevices : integer = 0;

//-----------------------------------------------------------------------------
// Name: UpdateInfoDlgText()
// Desc: Update all of the text and buttons in the dialog
//-----------------------------------------------------------------------------

procedure UpdateInfoDlgText( hDlg : HWND; iCurrent : integer; dwHost : DWORD );
var
  strBuffer : PChar;
  pDI : TDDDeviceIdentifier2;
  hNext, hPrev : HWND;
  Ver : ULarge_Integer;
begin
  if ( dwHost = DDGDI_GETHOSTIDENTIFIER ) then
    CheckRadioButton( hDlg, IDC_RADIO_DEVICE, IDC_RADIO_HOST, IDC_RADIO_HOST )
  else
    CheckRadioButton( hDlg, IDC_RADIO_DEVICE, IDC_RADIO_DEVICE, IDC_RADIO_DEVICE );

  pDI := g_DeviceIdent[ iCurrent ].DeviceInfo;
  if ( dwHost = DDGDI_GETHOSTIDENTIFIER ) then
    pDI := g_DeviceIdent[ iCurrent ].DeviceInfoHost;

  strBuffer := PChar( Format( 'Device information for device %d of %d', [ iCurrent + 1, g_iMaxDevices ] ) );
  SetDlgItemText( hDlg, IDC_RADIO_DEVICE, strBuffer );

    // Device ID stuff:
  strBuffer := PChar( Format( '%8.8x', [ pDI.dwVendorId ] ) );
  SetDlgItemText( hDlg, IDC_DWVENDORID, strBuffer );

  strBuffer := PChar( Format( '%8.8x', [ pDI.dwDeviceId ] ) );
  SetDlgItemText( hDlg, IDC_DWDEVICEID, strBuffer );

  strBuffer := PChar( Format( '%8.8x', [ pDI.dwSubSysId ] ) );
  SetDlgItemText( hDlg, IDC_DWSUBSYS, strBuffer );

  strBuffer := PChar( Format( '%8.8x', [ pDI.dwRevision ] ) );
  SetDlgItemText( hDlg, IDC_DWREVISION, strBuffer );

    // Driver version:
  Ver := ULarge_Integer( pDI.liDriverVersion );
  strBuffer := PChar( Format( '%d.%2.2d.%2.2d.%4.4d', [ HIWORD( Ver.HighPart ), LOWORD( Ver.HighPart ), HIWORD( Ver.LowPart ), LOWORD( Ver.LowPart ) ] ) );
  SetDlgItemText( hDlg, IDC_VERSION, strBuffer );

    // Device description and HAL filename
  SetDlgItemText( hDlg, IDC_DESCRIPTION, pDI.szDescription );
  SetDlgItemText( hDlg, IDC_FILENAME, pDI.szDriver );

    // Unique driver/device identifier:
  with pDI.guidDeviceIdentifier do
  begin
    strBuffer := PChar( Format( '{%8.8x-%4.4x-%4.4x-%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x%2.2x}',
      [ D1, D2, D3, D4[ 0 ], D4[ 1 ], D4[ 2 ], D4[ 3 ], D4[ 4 ], D4[ 5 ],
      D4[ 6 ], D4[ 7 ] ] ) );
  end;
  SetDlgItemText( hDlg, IDC_GUID, strBuffer );

    // WHQL Level
  strBuffer := PChar( Format( '%8.8x', [ pDI.dwWHQLLevel ] ) );
  SetDlgItemText( hDlg, IDC_STATIC_WHQLLEVEL, strBuffer );

    // Change the state and style of the Prev and Next buttons if needed
  hNext := GetDlgItem( hDlg, IDC_NEXT );
  hPrev := GetDlgItem( hDlg, IDC_PREV );

  if ( iCurrent = 0 ) then
  begin
        // The Prev button should be disabled
    SetFocus( GetDlgItem( hDlg, IDOK ) );
    SendDlgItemMessage( hDlg, IDC_PREV, BM_SETSTYLE, BS_PUSHBUTTON, 0 );
    SendMessage( hDlg, DM_SETDEFID, IDOK, 0 );
    EnableWindow( hPrev, FALSE );
  end
  else
  begin
    if ( IsWindowEnabled( hPrev ) = False ) then
      EnableWindow( hPrev, TRUE );
  end;

  if ( iCurrent >= ( g_iMaxDevices - 1 ) ) then
  begin
        // The Next button should be disabled
    SetFocus( GetDlgItem( hDlg, IDOK ) );
    SendDlgItemMessage( hDlg, IDC_NEXT, BM_SETSTYLE, BS_PUSHBUTTON, 0 );
    SendMessage( hDlg, DM_SETDEFID, IDOK, 0 );
    EnableWindow( hNext, FALSE );
  end
  else
  begin
    if ( IsWindowEnabled( hNext ) = False ) then
      EnableWindow( hNext, TRUE );
  end;
end;

//-----------------------------------------------------------------------------
// Name: InfoDlgProc()
// Desc: The dialog window proc
//-----------------------------------------------------------------------------

function InfoDlgProc( hDlg : HWND; amessage : UINT; wParm : WPARAM; lParam : LPARAM ) : Boolean; stdcall;
var
  iCurrent : integer;
  dwHost : DWORD;
begin
  iCurrent := 0;
  dwHost := 0;

  case ( amessage ) of
    WM_INITDIALOG :
      begin
      // Setup the first devices text
        UpdateInfoDlgText( hDlg, iCurrent, dwHost );
      end;

    WM_COMMAND :
      begin
        case ( LOWORD( wParm ) ) of
          IDOK :
            begin
              EndDialog( hDlg, 0 );
            end;

          IDCANCEL :
            begin
              EndDialog( hDlg, 0 );
            end;

          IDC_PREV :
            begin
          // Show the previous device
              if ( iCurrent <> 0 ) then
                iCurrent := iCurrent - 1;

              UpdateInfoDlgText( hDlg, iCurrent, dwHost );
            end;

          IDC_NEXT :
            begin
          // Show the next device
              if ( iCurrent < g_iMaxDevices ) then
                iCurrent := iCurrent + 1;

              UpdateInfoDlgText( hDlg, iCurrent, dwHost );
            end;

          IDC_RADIO_HOST :
            begin
              dwHost := DDGDI_GETHOSTIDENTIFIER;

              UpdateInfoDlgText( hDlg, iCurrent, dwHost );
            end;

          IDC_RADIO_DEVICE :
            begin
              dwHost := 0;

              UpdateInfoDlgText( hDlg, iCurrent, dwHost );
            end;

        else
          begin
            result := FALSE; // Message not handled
            exit;
          end;
        end;
      end;

  else
    begin
      result := FALSE; // Message not handled
      exit;
    end;
  end;

  result := TRUE; // Message handled
end;

//-----------------------------------------------------------------------------
// Name: DDEnumCallbackEx()
// Desc: This callback gets the information for each device enumerated
//-----------------------------------------------------------------------------

function DDEnumCallbackEx( pGUID : PGUID; pDescription : PAnsiChar; strName : PAnsiChar; pContext : Pointer; hm : HMONITOR ) : BOOL; stdcall;
var
  pDD : IDIRECTDRAW7;
  hr : HRESULT;
begin
  pDD := nil;

  // Create a DirectDraw object using the enumerated GUID
  hr := DirectDrawCreateEx( pGUID, pDD, IID_IDirectDraw7, nil );
  if ( hr <> DD_OK ) then
  begin
    result := LongBool( DDENUMRET_CANCEL );
    exit;
  end;

  // Get the device information and save it
  pDD.GetDeviceIdentifier( g_DeviceIdent[ g_iMaxDevices ].DeviceInfo, 0 );
  pDD.GetDeviceIdentifier( g_DeviceIdent[ g_iMaxDevices ].DeviceInfoHost, DDGDI_GETHOSTIDENTIFIER );

  // Finished with the DirectDraw object, so release it
  pDD := nil;

  // Bump to the next open slot or finish the callbacks if full
  if ( g_iMaxDevices < MAX_DEVICES ) then
    inc( g_iMaxDevices )
  else
  begin
    result := LongBool( DDENUMRET_CANCEL );
    exit;
  end;

  result := LongBool( DDENUMRET_OK );
end;

//-----------------------------------------------------------------------------
// Name: DDEnumCallback()
// Desc: Old style callback retained for backwards compatibility
//-----------------------------------------------------------------------------

function DDEnumCallback( p_GUID : pGUID; pDescription : PAnsiChar; strName : PAnsiChar; pContext : Pointer ) : BOOL; stdcall;
begin
  result := LongBool( DDEnumCallbackEx( p_GUID, pDescription, strName, pContext, 0 ) );
end;

var
  hr : HRESULT;
begin
  hr := DirectDrawEnumerateEx( DDEnumCallbackEx, nil, DDENUM_ATTACHEDSECONDARYDEVICES or
    DDENUM_DETACHEDSECONDARYDEVICES or
    DDENUM_NONDISPLAYDEVICES );
  if hr <> DD_OK then
  begin
    // try Old style enumeration if new style fails
    hr := DirectDrawEnumerate( DDEnumCallback, nil );
    if hr <> DD_OK then
    begin
      MessageBox( 0, 'Failed during enumeration.', TITLE, MB_OK or MB_ICONERROR );
      exit;
    end;
  end;

  if ( 0 = g_iMaxDevices ) then
  begin
    MessageBox( 0, 'No devices to enumerate.', TITLE, MB_OK or MB_ICONERROR );
  end;

  // Bring up the dialog to show all the devices
  DialogBox( hInstance, MAKEINTRESOURCE( IDD_DRIVERINFO ), GetDesktopWindow, @InfoDlgProc );
end.
