program DirectSurfaceWrite_api;
{******************************************************************}
{                                                                  }
{       Borland Delphi DirectX 8 SDK Examples                      }
{       Conversion of the Direct Draw Enumeration Example          }
{                                                                  }
{ Portions created by Microsoft are                                }
{ Copyright (C) 1995-2000 Microsoft Corporation.                   }
{ All Rights Reserved.                                             }
{                                                                  }
{ The original files are : DirectSurfaceWrite.h,                   }
{                          DirectSurfaceWrite.cpp                  }
{ The original Pascal code is : DirectSurfaceWrite_api.pas         }
{ The initial developer of the Pascal code is : Dominique Louis    }
{ ( Dominique@SavageSoftware.com.au )                              }
{                                                                  }
{ Portions created by Dominique Louis are                          }
{ Copyright (C) 2000 - 2001 Dominique Louis.                       }
{                                                                  }
{ Contributor(s)                                                   }
{ --------------                                                   }
{ <Contributer Name> ( contributer@sp.sw )                         }
{                                                                  }
{ Obtained through:                                                }
{ Joint Endeavour of Delphi Innovators ( Project JEDI )            }
{                                                                  }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{                                                                  }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/NPL/NPL-1_1Final.html                     }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{ Description                                                      }
{ -----------                                                      }
{   This sample demonstrates how to animate sprites using          }
{   DirectDraw.  The samples runs in full-screen mode.  Pressing   }
{   any key will exit the sample.                                  }
{                                                                  }
{ Requires                                                         }
{ --------                                                         }
{   Eric Unger and Tim Baumgarten's conversions of the DirectX     }
{   headers. They are available from...                            }
{   http://delphi-jedi.org/DelphiGraphics/ .                       }
{                                                                  }
{                                                                  }
{ Programming Notes                                                }
{ -----------------                                                }
{   For details on how to setup a full-screen DirectDraw app, see  }
{   the FullScreenMode sample.                                     }
{                                                                  }
{   To write directly on a DirectDraw surface first call           }
{   IDirectDrawSurface::Lock to obtain a pointer directly into the }
{   memory of the DirectDraw surface.  While the surface is locked,}
{   the surface can not be blted or flipped onto other surfaces.   }
{   The surface's pixel format will tell you the data format the   }
{   surface stores pixels in.  Be sure to advance the surface      }
{   pointer by the surface pitch instead of the surface            }
{   width, since the surface may be wider than its width.          }
{   After the drawing is finished call IDirectDrawSurface::Unlock  }
{   to allow the surface to blt to other surfaces.  See            }
{   DrawSprite() in this sample for an example of how this is done.}
{                                                                  }
{ Revision History                                                 }
{ ----------------                                                 }
{   2000-12-01 - DL  Initial translation.                          }
{                                                                  }
{                                                                  }
{******************************************************************}

uses
  Windows,
  SysUtils,
  Messages,
  DirectDraw,
  DDUtil in '..\..\Common\DDUtil.pas';

{$R *.RES}
const
  NAME : PChar = 'DirectSurfaceWrite';
  TITLE : PChar = 'DirectDraw Direct Surface Write Example';
  HELPTEXT : PChar = 'Press Escape to quit.';

  IDI_MAIN_ICON = 101;
  IDI_MAIN = 102;
  IDR_MAIN_ACCEL = 103;
  IDM_EXIT = 1001;

  SCREEN_WIDTH = 640;
  SCREEN_HEIGHT = 480;
  SCREEN_BPP = 16;

  SPRITE_DIAMETER = 250;
  NUM_SPRITES = 5;



type
  PSpriteRecord = ^TSpriteRecord;
  TSpriteRecord = packed record
    fPosX : single;
    fPosY : single;
    fVelX : single;
    fVelY : single;
  end;

var
  g_pDisplay : TDisplay = nil;
  g_pSpriteSurface : TSurface = nil;
  g_pTextSurface : TSurface = nil;
  g_hWnd : HWND = 0;
  g_bActive : Boolean = False; // Is application active?
  g_dwLastTick : DWord;
  g_Sprite : array[ 0..NUM_SPRITES - 1 ] of TSpriteRecord;


{-----------------------------------------------------------------------------
  Name: FreeDirectDraw
  Desc: Release all the DirectDraw objects
-----------------------------------------------------------------------------}

procedure FreeDirectDraw;
begin
  g_pSpriteSurface.Free;
  g_pSpriteSurface := nil;
  g_pTextSurface.Free;
  g_pTextSurface := nil;
  g_pDisplay.Free;
  g_pDisplay := nil;
end;

{-----------------------------------------------------------------------------
   Name: UpdateSprite()
   Desc: Move the sprite around and make it bounce based on how much time
         has passed
-----------------------------------------------------------------------------}

procedure UpdateSprite( var pSprite : TSpriteRecord; fTimeDelta : Single );
begin
  // Update the sprite position
  pSprite.fPosX := pSprite.fPosX + pSprite.fVelX * fTimeDelta;
  pSprite.fPosY := pSprite.fPosY + pSprite.fVelY * fTimeDelta;

  // Clip the position, and bounce if it hits the edge
  if ( pSprite.fPosX < 0.0 ) then
  begin
    pSprite.fPosX := 0;
    pSprite.fVelX := -pSprite.fVelX;
  end;

  if ( pSprite.fPosX >= SCREEN_WIDTH - SPRITE_DIAMETER ) then
  begin
    pSprite.fPosX := SCREEN_WIDTH - 1 - SPRITE_DIAMETER;
    pSprite.fVelX := -pSprite.fVelX;
  end;

  if ( pSprite.fPosY < 0 ) then
  begin
    pSprite.fPosY := 0;
    pSprite.fVelY := -pSprite.fVelY;
  end;

  if ( pSprite.fPosY > SCREEN_HEIGHT - SPRITE_DIAMETER ) then
  begin
    pSprite.fPosY := SCREEN_HEIGHT - 1 - SPRITE_DIAMETER;
    pSprite.fVelY := -pSprite.fVelY;
  end;
end;

{-----------------------------------------------------------------------------
   Name: DisplayFrame()
   Desc: Blts a the sprites to the back buffer, then it blts or flips the
         back buffer onto the primary buffer.
-----------------------------------------------------------------------------}

function DisplayFrame : HRESULT;
var
  hr : HRESULT;
  iSprite : integer;
begin
  if g_pDisplay = nil then
  begin
    result := DD_OK;
    exit;
  end;

  // Fill the back buffer with black, ignoring errors until the flip
  g_pDisplay.Clear( 0 );

  // Blt the help text on the backbuffer, ignoring errors until the flip
  g_pDisplay.Blt( 10, 10, g_pTextSurface, nil );

  // Blt all the sprites onto the back buffer using color keying,
  // ignoring errors until the flip. Note that all of these sprites
  // use the same DirectDraw surface.
  for iSprite := 0 to NUM_SPRITES - 1 do
    g_pDisplay.Blt( Round( g_Sprite[ iSprite ].fPosX ), Round( g_Sprite[ iSprite ].fPosY ), g_pSpriteSurface, nil );


  // Flip or blt the back buffer onto the primary buffer
  hr := g_pDisplay.Flip;
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: DrawSprite()
// Desc: Draws a pattern of colors to a DirectDraw surface by directly writing
//       to the surface memory.  This function was designed to work only in
//       16-bit color.
//-----------------------------------------------------------------------------

function DrawSprite : HRESULT;
var
  ddsd : TDDSURFACEDESC2;
  hr : HRESULT;
  dwShift, dwBits, dwRed, dwGreen, dwBlue : DWORD;
  fPercentX, fPercentY, fPercentXY : Single;
  pDDS : IDIRECTDRAWSURFACE7;
  pDDSColor : PWORD;
  iY, iX : DWord;
begin
  pDDS := g_pSpriteSurface.GetDDrawSurface;

  FillChar( ddsd, SizeOf( ddsd ), 0 ); //ZeroMemory( @ddsd,sizeof(ddsd) );
  ddsd.dwSize := SizeOf( ddsd );

  // Lock the surface to directly write to the surface memory
  Result:= pDDS.Lock( nil, ddsd, DDLOCK_WAIT, 0 );
  if FAILED(Result) then Exit;

  // Get a pointer into the memory starting at ddsd.lpSurface.  Cast this pointer to a
  // 16-bit WORD since we are in 16 bit color, so each pixel has 16 bits of color information.
  pDDSColor := PWord(ddsd.lpSurface);

  for iY := 0 to ddsd.dwHeight - 1 do
  begin
    for iX := 0 to ddsd.dwWidth - 1 do
    begin
      // Figure out the red component as a function of the Y location of the pixel
      g_pSpriteSurface.GetBitMaskInfo(ddsd.ddpfPixelFormat.dwRBitMask, dwShift, dwBits);
      fPercentY := abs( ddsd.dwHeight div 2 - iY ) / ( ( ddsd.dwHeight div 2 ) + 0.25 );
      if ( fPercentY > 1.00 ) then
        fPercentY := 1.00;
      dwRed := Trunc((ddsd.ddpfPixelFormat.dwRBitMask shr dwShift) * fPercentY) shl dwShift;

      // Figure out the green component as a function of the X location of the pixel
      g_pSpriteSurface.GetBitMaskInfo( ddsd.ddpfPixelFormat.dwGBitMask, dwShift, dwBits );
      fPercentX := abs( ddsd.dwWidth div 2 - iX ) / ( ( ddsd.dwWidth div 2 ) + 0.25 );
      if ( fPercentX > 1.00 ) then
        fPercentX := 1.00;
      dwGreen := Round( (ddsd.ddpfPixelFormat.dwGBitMask shr dwShift) * fPercentX) shl dwShift;

      // Figure out the blue component as a function of the X and Y location of the pixel
      g_pSpriteSurface.GetBitMaskInfo( ddsd.ddpfPixelFormat.dwBBitMask, dwShift, dwBits );
      fPercentX := abs( ddsd.dwWidth / 2 - iX ) / ( ddsd.dwWidth / 4 );
      fPercentX := 1.0 - fPercentX * fPercentX;
      fPercentY := abs( ddsd.dwHeight / 2 - iY ) / ( ddsd.dwHeight / 4 );
      fPercentY := 1.0 - fPercentY * fPercentY;
      fPercentXY := fPercentX + fPercentY;
      if ( fPercentXY > 1.0 ) then
        fPercentXY := 1.0;
      if ( fPercentXY < 0.0 ) then
        fPercentXY := 0.0;
      dwBlue := Trunc((ddsd.ddpfPixelFormat.dwBBitMask shr dwShift) * fPercentXY ) shl dwShift;

      // Make the dwDDSColor by combining all the color components
      pDDSColor^ := Word( dwRed or dwGreen or dwBlue );

      // Advance the surface pointer by 16 bits (one pixel)
      Inc(pDDSColor);
    end;

    // Multiply ddsd.lPitch by iY to figure out offset needed to access
    // the next scan line on the surface.
    pDDSColor := PWord(LongWord(ddsd.lpSurface) + (iY + 1)*LongWord(ddsd.lPitch));
  end;

  // Unlock the surface
  pDDS.Unlock( nil );

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: RestoreSurfaces()
   Desc: Restore all the surfaces, and redraw the sprite surfaces.
-----------------------------------------------------------------------------}

function RestoreSurfaces : HRESULT;
var
  hr : HRESULT;
  pDDPal : IDIRECTDRAWPALETTE;
begin
  pDDPal := nil;

  hr := g_pDisplay.GetDirectDraw.RestoreAllSurfaces;
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( TEXT("RestoreAllSurfaces"), hr );
    result := hr;
    exit;
  end;

  // No need to re-create the surface, just re-draw it.
  hr := g_pTextSurface.DrawText( 0, HELPTEXT, 0, 0, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( TEXT("DrawText"), hr );
    result := hr;
    exit;
  end;

  // No need to re-create the surface, just re-draw it.
  hr := DrawSprite;
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: ProcessNextFrame()
   Desc: Move the sprites, blt them to the back buffer, then
         flip or blt the back buffer to the primary buffer
-----------------------------------------------------------------------------}

function ProcessNextFrame( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
  dwCurrTick : DWord;
  dwTickDiff : DWord;
  iSprite : integer;
begin
  // Figure how much time has passed since the last time
  dwCurrTick := GetTickCount;
  dwTickDiff := dwCurrTick - g_dwLastTick;

  // Don't update if no time has passed
  if ( dwTickDiff = 0 ) then
  begin
    result := DD_OK;
    exit
  end;

  g_dwLastTick := dwCurrTick;

  // Move the sprites according to how much time has passed
  for iSprite := 0 to NUM_SPRITES - 1 do
    UpdateSprite( g_Sprite[ iSprite ], dwTickDiff / 1000.0 );

  // Display the sprites on the screen
  hr := DisplayFrame;
  if ( hr <> DD_OK ) then
  begin
    if ( hr <> DDERR_SURFACELOST ) then
    begin
      FreeDirectDraw;
      //DXTRACE_ERR( 'DisplayFrame', hr );
      result := hr;
      exit;
    end;

    // The surfaces were lost so restore them
    RestoreSurfaces;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: WindowProc
   Desc: The Main Window Procedure
{-----------------------------------------------------------------------------}

function WindowProc( h_Wnd : THandle; aMSG : Cardinal; wParam : Cardinal; lParam : Integer ) : Integer; stdcall;
begin
  case aMSG of
    // Pause if minimized
    WM_COMMAND :
      begin
        case LOWORD( wParam ) of
          IDM_EXIT :
            begin
          // Received key/menu command to exit app
              PostMessage( h_Wnd, WM_CLOSE, 0, 0 );
              result := 0;
              exit;
            end;
        end;
      end;

    WM_SETCURSOR :
      begin
      // Hide the cursor if in fullscreen
        SetCursor( 0 );
        result := 1;
        exit;
      end;

    WM_SIZE :
      begin
      // Check to see if we are losing our window...
        if ( wParam = SIZE_MAXHIDE ) or ( wParam = SIZE_MINIMIZED ) then
          g_bActive := FALSE
        else
          g_bActive := TRUE;
      end;

    WM_EXITMENULOOP :
      begin
      // Ignore time spent in menu
        g_dwLastTick := GetTickCount;
      end;

    WM_EXITSIZEMOVE :
      begin
      // Ignore time spent resizing
        g_dwLastTick := GetTickCount;
      end;

    WM_SYSCOMMAND :
      begin
      // Prevent moving/sizing and power loss in fullscreen mode
        case wParam of
          SC_MOVE, SC_SIZE, SC_MAXIMIZE, SC_MONITORPOWER :
            begin
              result := 1;
              exit;
            end;
        end;
      end;

    WM_DESTROY :
      begin
      // Cleanup and close the app
        FreeDirectDraw;
        PostQuitMessage( 0 );
        result := 0;
        exit;
      end;
  end;
  Result := DefWindowProc( h_Wnd, aMSG, wParam, lParam );
end;

{-----------------------------------------------------------------------------
   Name: WinInit
   Desc: Do work required for every instance of the application:
         Create the window, initialize data
-----------------------------------------------------------------------------}

function WinInit( h_Inst : THANDLE; nCmdShow : Integer; var phWnd : THandle; var phAccel : HACCEL ) : HRESULT;
var
  h_Wnd : THandle;
  wc : TWNDCLASS;
  h_Accel : HACCEL;
begin
  // Register the window class
  wc.lpszClassName := NAME;
  wc.lpfnWndProc := @WindowProc;
  wc.style := CS_VREDRAW or CS_HREDRAW;
  wc.hInstance := h_Inst;
  wc.hIcon := LoadIcon( h_Inst, MakeIntResource( IDI_MAIN_ICON ) );
  wc.hCursor := LoadCursor( 0, IDC_ARROW );
  wc.hbrBackground := ( COLOR_WINDOW + 1 );
  wc.lpszMenuName := nil;
  wc.cbClsExtra := 0;
  wc.cbWndExtra := 0;

  if ( RegisterClass( wc ) = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  // Load keyboard accelerators
  h_Accel := LoadAccelerators( h_Inst, MakeIntResource( IDR_MAIN_ACCEL ) );

  // Create and show the main window
  h_Wnd := CreateWindowEx( 0,
    NAME,
    TITLE,
    WS_POPUP,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    0,
    0,
    h_Inst,
    nil );
  if ( h_Wnd = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  ShowWindow( h_Wnd, nCmdShow );
  UpdateWindow( h_Wnd );

  phWnd := h_Wnd;
  phAccel := h_Accel;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: InitDirectDraw()
   Desc: Create the DirectDraw object, and init the surfaces
-----------------------------------------------------------------------------}

function InitDirectDraw( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
  iSprite : integer;
begin
  // Initialize all the surfaces we need
  g_pDisplay := TDisplay.Create;

  hr := g_pDisplay.CreateFullScreenDisplay( h_Wnd, SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'This display card does not support 640x480x16.', TITLE, MB_ICONERROR or MB_OK );
    result := hr;
    exit;
  end;

  // Create a surface, and draw a bitmap resource on it.
  hr := g_pDisplay.CreateSurface( g_pSpriteSurface, SPRITE_DIAMETER, SPRITE_DIAMETER );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'CreateSurface', hr )
    result := hr;
    exit;
  end;

  hr := DrawSprite;
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'DrawSprite', hr )
    result := hr;
    exit;
  end;

  // Create a surface, and draw text to it.
  hr := g_pDisplay.CreateSurfaceFromText( g_pTextSurface, 0, HELPTEXT, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'CreateSurfaceFromText', hr )
    result := hr;
    exit
  end;

  // Init all the sprites.  All of these sprites look the same,
  // using the g_pDDSLogo surface.
  for iSprite := 0 to NUM_SPRITES - 1 do
  begin
    // Set the sprite's position and velocity
    g_Sprite[ iSprite ].fPosX := random( SCREEN_WIDTH );
    g_Sprite[ iSprite ].fPosY := random( SCREEN_HEIGHT );

    g_Sprite[ iSprite ].fVelX := 500.0 * random / RAND_MAX - 250.0;
    g_Sprite[ iSprite ].fVelY := 500.0 * random / RAND_MAX - 250.0;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: WinMain
   Desc: Entry point to the program. Initializes everything and calls
         UpdateFrame() when idle from the message pump.
-----------------------------------------------------------------------------}
var
  aMSG : MSG;
  h_Wnd : THandle;
  h_Accel : HACCEL;
  hr : HRESULT;
begin
  FillChar( g_Sprite, SizeOf( TSpriteRecord ) * NUM_SPRITES, 0 );

  random( GetTickCount );
  hr := WinInit( hInstance, SW_SHOW, h_Wnd, h_Accel );
  if ( hr <> DD_OK ) then
    exit;

  hr := InitDirectDraw( h_Wnd );
  if ( hr <> DD_OK ) then
  begin
    //DXTRACE_ERR( 'InitDirectDraw', hr );
    MessageBox( h_Wnd, 'DirectDraw init failed. ' + #13#10 + 'The sample will now exit. ', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
    exit;
  end;

  g_dwLastTick := GetTickCount;

  while ( true ) do
  begin
    // Look for messages, if none are found then
    // update the state and display it
    if ( PeekMessage( aMSG, 0, 0, 0, PM_NOREMOVE ) ) then
    begin
      if not GetMessage( aMSG, 0, 0, 0 ) then
        // WM_QUIT was posted, so exit
        exit;

      // Translate and dispatch the message
      if ( TranslateAccelerator( h_Wnd, h_Accel, aMsg ) = 0 ) then
      begin
        TranslateMessage( aMsg );
        DispatchMessage( aMsg );
      end;
    end
    else
    begin
      if ( g_bActive ) then
      begin
        // Move the sprites, blt them to the back buffer, then
        // flip or blt the back buffer to the primary buffer
        hr := ProcessNextFrame( h_Wnd );
        if ( hr <> DD_OK ) then
        begin
          g_pDisplay := nil;
          if ( hr = E_NOTIMPL ) then
          begin
            MessageBox( h_Wnd, 'The driver does not support waiting for vertical blank.' + #13#10 +
              'This is expected if the HW acceleration is disabled.' + #13#10 +
              'The sample will now exit.', TITLE, MB_ICONERROR or MB_OK );
          end
          else
          begin
              //DXTRACE_ERR( 'ProcessNextFrame', hr );
            MessageBox( h_Wnd, 'Displaying the next frame failed.'
              + #13#10 + 'The sample will now exit.',
              TITLE, MB_ICONERROR or MB_OK );
          end;
          exit;
        end;
      end
      else
      begin
        // Go to sleep if we have nothing else to do
        WaitMessage;

        // Ignore time spent inactive
        g_dwLastTick := GetTickCount;
      end;
    end;
  end;
end.
