program RefreshRate_api;
{******************************************************************}
{                                                                  }
{       Borland Delphi DirectX 8 SDK Examples                      }
{       Conversion of the Overlay Animate Example                  }
{                                                                  }
{ Portions created by Microsoft are                                }
{ Copyright (C) 1995-2000 Microsoft Corporation.                   }
{ All Rights Reserved.                                             }
{                                                                  }
{ The original files are : RefreshRate.h,                          }
{                          RefreshRate.cpp                         }
{ The original Pascal code is : OverlayAnimate_vcl.pas             }
{ The initial developer of the Pascal code is : Dominique Louis    }
{ ( Dominique@SavageSoftware.com.au )                              }
{                                                                  }
{ Portions created by Dominique Louis are                          }
{ Copyright (C) 2000 - 2001 Dominique Louis.                       }
{                                                                  }
{ Contributor(s)                                                   }
{ --------------                                                   }
{ <Contributer Name> ( contributer@sp.sw )                         }
{                                                                  }
{ Obtained through:                                                }
{ Joint Endeavour of Delphi Innovators ( Project JEDI )            }
{                                                                  }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{                                                                  }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/NPL/NPL-1_1Final.html                     }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{ Description                                                      }
{ -----------                                                      }
{    This example demonstrates basic usage of the IDirectDraw7::StartRefreshRate
    and IDirectDraw7::EvaluateMode methods. Together, these methods allow an
    application to explore what display modes and refresh rates the monitor
    connected to this display device is able to display, though a manual
    user-controlled process. The application will present the UI that asks
    the user if the current mode under test is being displayed correctly by
    the monitor.

    Applications should use these methods when they are interested in using
    higher refresh rates.    }
{                                                                  }
{ Requires                                                         }
{ --------                                                         }
{   Eric Unger and Tim Baumgarten's conversions of the DirectX     }
{   headers. They are available from...                            }
{   http://delphi-jedi.org/DelphiGraphics/ .                       }
{                                                                  }
{                                                                  }
{ Programming Notes                                                }
{ -----------------                                                }
{   The basic idea is that DirectDraw will setup a list of modes to be tested
    (based on the list the app passed in), and then sequentially test them
    under application control. The app starts the test process, and then
    calls IDirectDraw7::EvaluateMode continuously. DirectDraw will take care
    of settings the modes. All the app has to do is SetCooperativeLevel
    beforehand, and then handle surface loss and drawing the UI that asks the
    user if they can see the current mode under test. DirectDraw returns
    enough information from IDirectDraw7::EvalulateMode to allow the app to
    know when to do these things, and when to stop testing. The app can pass
    a flag to IDirectDraw7::EvaluateMode if the user happened to say they
    could see the mode corretly, which will cause DirectDraw to mark the mode
    as good and move on. DirectDraw may also decide that time as run out and
    give up on a certain mode.

    DirectDraw uses information at its disposal from any automated means to
    make the testing process as short as possible, and applications only need
    to test modes they are interested in.}
{                                                                  }
{ Revision History                                                 }
{ ----------------                                                 }
{   2000-12-01 - DL  Initial translation.                          }
{                                                                  }
{                                                                  }
{******************************************************************}

uses
  Windows,
  SysUtils,
  Messages,
  DirectDraw;

{$R *.RES}
const
  NAME  : PChar = 'RefreshRate';
  TITLE : PChar = 'DirectDraw Refresh Rate Example';

  IDI_MAIN_ICON                   = 101;
  IDD_MAIN                        = 102;
  IDR_MAIN_ACCEL                  = 103;
  IDC_DDRAW_DEVICE_LIST           = 1000;
  IDC_TEST_MODES_LIST             = 1001;
  IDC_ALL_MODES_LIST              = 1002;
  IDC_CLOSE                       = 1017;
  IDC_EXIT                        = 2;
  IDC_TEST                        = 3;
  IDC_RESET                       = 4;

  MAX_DEVICES                     = 16;
  MAX_MODESIZES                   = 256;


type
  TDDrawDeviceInfo = packed record
    FGuid : TGUID;
    strDescription : string;
    strDriverName : string;
    dwModeCount : DWord;
    aModeSize : array[ 0..MAX_MODESIZES - 1 ] of TSize;
  end;

var
  g_hDlg : HWND;
  g_aDevices : array[ 0..MAX_DEVICES - 1 ] of TDDrawDeviceInfo;
  g_dwDeviceCount : DWord;

//-----------------------------------------------------------------------------
// Name: EnumAllModesCallback()
// Desc: For each mode enumerated, it adds it to the "All Modes" listbox.
//-----------------------------------------------------------------------------
function EnumAllModesCallback( const pddsd : TDDSURFACEDESC2; pContext : Pointer ) :HResult; stdcall; //RZ Added Const
var
  h_Wnd : HWND;
begin
  h_Wnd := HWND( pContext );

  SendMessage( h_Wnd, LB_ADDSTRING, 0, LPARAM( Format( '%dx%dx%d - %d Hz',[
                              pddsd.dwWidth,
                              pddsd.dwHeight,
                              pddsd.ddpfPixelFormat.dwRGBBitCount,
                              pddsd.dwRefreshRate] ) ) );

  result := DDENUMRET_OK;
end;

//-----------------------------------------------------------------------------
// Name: UpdateModesListBox()
// Desc: Updates the "modes to test" and "all modes" list boxes
//-----------------------------------------------------------------------------
function UpdateModesListBox( dwDeviceIndex : DWORD ) : HRESULT;
var
  pDD : IDIRECTDRAW7;
  hr : HRESULT;
  hWndModesToTest, hWndAllModes : HWND;
  i : DWord;
begin
  pDD := nil;

  hWndModesToTest := GetDlgItem( g_hDlg, IDC_TEST_MODES_LIST );
  SendMessage( hWndModesToTest, LB_RESETCONTENT, 0, 0 );

  // Update the "modes to test" list box based on the display device selected
  for i := 0 to g_aDevices[dwDeviceIndex].dwModeCount - 1 do
  begin
    // Make a string based on the this mode's size and
    // Add it to the list box
    SendMessage( hWndModesToTest, LB_ADDSTRING, 0, LPARAM( Format('%d x %d',[g_aDevices[dwDeviceIndex].aModeSize[i].cx, g_aDevices[dwDeviceIndex].aModeSize[i].cy]) ) );
    SendMessage( hWndModesToTest, LB_SETITEMDATA, i, LPARAM( i ) );
  end;

  // Create a DirectDraw device based using the selected device guid
  hr := DirectDrawCreateEx( @g_aDevices[dwDeviceIndex].FGuid, pDD, IID_IDirectDraw7, nil );
  if( hr = DD_OK ) then
  begin
    hWndAllModes := GetDlgItem( g_hDlg, IDC_ALL_MODES_LIST );
    SendMessage( hWndAllModes, LB_RESETCONTENT, 0, 0 );

    // Enumerate and display all supported modes along
    // with supported bit depth, and refresh rates
    // in the "All Modes" listbox
    hr := pDD.EnumDisplayModes( DDEDM_REFRESHRATES, nil,  Pointer( hWndAllModes ), @EnumAllModesCallback );

    // Release this device
    pDD := nil;
  end;

  result := hr;
end;

//-----------------------------------------------------------------------------
// Name: OnInitDialog()
// Desc: Initializes the dialogs (sets up UI controls, etc.)
//-----------------------------------------------------------------------------
procedure OnInitDialog;
var
  h_Inst : HInst;
  h_Icon : HICON;
  hWndDeviceList : HWND;
  i : UINT;
  dwCurrentSelect : DWord;
  hr : Hresult;
begin
  // Load the icon
{$ifdef __WIN64}
  h_Inst := GetWindowLongPtr( g_hDlg, GWLP_HINSTANCE );
{$else}
  h_Inst := GetWindowLong( g_hDlg, GWL_HINSTANCE );
{$endif}
  h_Icon := LoadIcon( h_Inst, MAKEINTRESOURCE( IDI_MAIN_ICON ) );

  // Set the icon for this dialog.
  PostMessage( g_hDlg, WM_SETICON, ICON_BIG,   LPARAM( h_Icon ) );  // Set big icon
  PostMessage( g_hDlg, WM_SETICON, ICON_SMALL, LPARAM( h_Icon ) );  // Set small icon

  // Show all available DirectDraw devices in the listbox
  hWndDeviceList := GetDlgItem( g_hDlg, IDC_DDRAW_DEVICE_LIST );
  for i := 0 to  g_dwDeviceCount -1 do
  begin
    SendMessage( hWndDeviceList, LB_ADDSTRING, 0, LPARAM( g_aDevices[i].strDescription ) );
    SendMessage( hWndDeviceList, LB_SETITEMDATA, i, LPARAM( i ) );
  end;

  // Select the first device by default
  dwCurrentSelect := 0;
  SendMessage( hWndDeviceList, LB_SETCURSEL, dwCurrentSelect, 0);

  hr := UpdateModesListBox( dwCurrentSelect );
  if( hr <> S_OK ) then
  begin
    MessageBox( g_hDlg, 'Error enumerating DirectDraw modes.' + #13#10 + 'Sample will now exit.', 'DirectDraw Refresh Rete Example', MB_OK or MB_ICONERROR );
    EndDialog( g_hDlg, IDABORT );
  end;

  SetFocus( hWndDeviceList );
end;

//-----------------------------------------------------------------------------
// Name: SetupPrimarySurface()
// Desc: Setups a primary DirectDraw surface
//-----------------------------------------------------------------------------
function SetupPrimarySurface( pDD : IDIRECTDRAW7; var ppDDS : IDIRECTDRAWSURFACE7 ) : HRESULT;
var
  ddsd : TDDSURFACEDESC2; 
begin
  FillChar( ddsd, SizeOf(ddsd), 0 );

  ddsd.dwSize         := sizeof(ddsd);
  ddsd.dwFlags        := DDSD_CAPS;
  ddsd.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE;

  result := pDD.CreateSurface( ddsd, ppDDS, nil );
end;

//-----------------------------------------------------------------------------
// Name: UpdatePrimarySurface()
// Desc: Fills the primary surface with white, and diplays the timeout value
//       on screen
//-----------------------------------------------------------------------------
function UpdatePrimarySurface( pDDS : IDIRECTDRAWSURFACE7; dwTimeout : DWORD  ) : HRESULT;
var
  ddbltfx : TDDBLTFX ;
  h_DC : HDC;
  strTimeout : string;
  rect : TRECT;
  hr : HRESULT;
begin
  // Clear the screen:
  FillChar( ddbltfx, SizeOf(ddbltfx), 0 );
  ddbltfx.dwSize      := sizeof(ddbltfx);
  ddbltfx.dwFillColor := $FFFFFFFF;

  hr := pDDS.Blt( nil, nil, nil, DDBLT_COLORFILL or DDBLT_WAIT, @ddbltfx );
  if( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  // Display the timeout value
  hr := pDDS.GetDC( h_DC );
  if( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  GetWindowRect( g_hDlg, rect );
  strTimeout := Format( 'Press space to accept or escape to reject.' + #13#10 +
          '%2d seconds until timeout', [dwTimeout] );
  DrawText( h_DC, PChar( strTimeout ), length(strTimeout), rect, DT_CENTER or DT_VCENTER or DT_SINGLELINE );

  // Cleanup
  pDDS.ReleaseDC( h_DC );

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: PerformDirectDrawRefreshRate()
// Desc: Perform the IDirectDraw7::StartRefreshRate and IDirectDraw7::EvaluateMode
//       tests
// Returns: S_OK if no modes needed testing, or all modes tested successfully,
//          informative error code otherwise.
//-----------------------------------------------------------------------------
function PerformDirectDrawRefreshRate( pDD : IDIRECTDRAW7; var aTestModes : array of TSize; dwTestModes : DWORD) : HRESULT;
var
  pDDSPrimary : IDIRECTDRAWSURFACE7;
  hr : HRESULT;
  AMsg : MSG;
  dwFlags : DWORD;
  dwTimeout : DWORD;
  bMsgReady : Boolean;
begin
  pDDSPrimary := nil;
  dwFlags := 0;
  // First call StartRefreshRate with the DDSMT_ISTESTREQUIRED flag to determine
  // whether the tests can be performed and need to be performed.
  hr := pDD.StartModeTest( aTestModes, dwTestModes, DDSMT_ISTESTREQUIRED);

  case hr of
    DDERR_NEWMODE:
    begin
    end;

    DDERR_TESTFINISHED:
    begin
      // DDERR_TESTFINISHED means that all the modes that we wish to test have already been tested correctly
     result := DD_OK;
     exit;
    end;
    
    else
    begin
      //Possible return codes here include DDERR_NOMONITORINFORMATION or DDERR_NODRIVERSUPPORT or
      //other fatal error codes (DDERR_INVALIDPARAMS, DDERR_NOEXCLUSIVEMODE, etc.)
      result := hr;
      exit;
    end;
  end;

  hr := pDD.StartModeTest( aTestModes, dwTestModes, 0 );
  if( hr = DDERR_TESTFINISHED ) then
  begin
    // The tests completed early, so return
    result := DD_OK;
    exit;
  end;

  // Create the primary DirectDraw surface
  hr := SetupPrimarySurface( pDD, pDDSPrimary );
  if( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  // Loop until the mode tests are complete
  while( TRUE ) do
  begin
    bMsgReady := PeekMessage( AMsg, 0, 0, 0, PM_REMOVE);

    if( bMsgReady ) then
    begin
      if (AMsg.message = WM_KEYDOWN) then
      begin
        case AMsg.wParam of
          VK_SPACE:
            dwFlags := DDEM_MODEPASSED;

          VK_ESCAPE:
            dwFlags := DDEM_MODEFAILED;
        end;
      end
      else
      begin
        TranslateMessage( AMsg );
        DispatchMessage( AMsg );
      end;
    end
    else
    begin
      // This method will only succeed with monitors that contain EDID data.
      // If the monitor is not EDID compliant, then the method will return
      // DDERR_TESTFINISHED without testing any modes. If the EDID table does not
      // contain values higher than 60hz, no modes will tested. Refresh rates
      // higher than 100 hz will only be tested if the EDID table contains values
      // higher than 85hz.
      hr := pDD.EvaluateMode(dwFlags, dwTimeout);

      case hr of

        DD_OK :
        begin
          if( pDDSPrimary <> nil ) then
          begin
            // Clear the screen, and display the timeout value
            UpdatePrimarySurface( pDDSPrimary, dwTimeout );
          end;
        end;

        DDERR_NEWMODE :
        begin
          // Cleanup the last DirectDraw surface, and create
          // a new one for the new mode
          pDDSPrimary := nil;

          SetupPrimarySurface( pDD, pDDSPrimary );
          if( hr <>DD_OK ) then
          begin
            result := hr;
            exit;
          end;

          dwFlags := 0;
        end;

        DDERR_TESTFINISHED :
        begin
          // Test complete, so stop looping
          break;
        end;
      end;
      Sleep( 100 );
    end;
  end;

  // Cleanup
  pDDSPrimary := nil;

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: OnRefreshRate()
// Desc: User hit the "Test" button
//-----------------------------------------------------------------------------
function OnRefreshRate : HRESULT;
var
  hWndModesToTest : HWND;
  dwSelectCount : DWORD;
  pDD : IDIRECTDRAW7;
  hr : HRESULT;
  hWndDeviceList : HWND;
  dwDeviceIndex : DWORD;
  i : DWORD;
  aTestModes : array[ 0..256 - 1 ] of TSize;
  dwTestModes : DWord; 
begin
  hWndModesToTest := GetDlgItem( g_hDlg, IDC_TEST_MODES_LIST );
  dwSelectCount := SendMessage( hWndModesToTest, LB_GETSELCOUNT, 0, 0 );

  if( dwSelectCount > 0 ) then
  begin
    pDD := nil;
    // Get the currently selected DirectDraw device
    hWndDeviceList := GetDlgItem( g_hDlg, IDC_DDRAW_DEVICE_LIST );
    dwDeviceIndex := SendMessage( hWndDeviceList, LB_GETCURSEL, 0, 0 );

    // Create a DirectDraw device based using the selected device guid
    hr := DirectDrawCreateEx( @g_aDevices[dwDeviceIndex].FGuid, pDD, IID_IDirectDraw7, nil);
    if( hr <> DD_OK ) then
    begin
      result := hr;
      exit;
    end;

    // This is a good usage of DDSCL_CREATEDEVICEWINDOW: DirectDraw will create a window that covers
    // the monitor, and won't mess around with our dialog box. Any mouse clicks on the cover window
    // will therefore not be received and misinterpreted by the dialog box, since such clicks will
    // be sent to DirectDraw's internal message procedure and therein ignored.
    hr := pDD.SetCooperativeLevel( g_hDlg,
                                           DDSCL_EXCLUSIVE          or
                                           DDSCL_FULLSCREEN         or
                                           DDSCL_CREATEDEVICEWINDOW or
                                           DDSCL_SETFOCUSWINDOW );
    if( hr <> DD_OK ) then
    begin
      pDD := nil;
      result := hr;
      exit;
    end;

    dwTestModes := 0;

    // Find out which modes are selected, then just test those
    for i := 0 to g_aDevices[dwDeviceIndex].dwModeCount - 1 do
    begin
      if( SendMessage( hWndModesToTest, LB_GETSEL, i, 0 ) <> LB_ERR ) then
      begin
        // Record the selected modes in aTestModes[]
        aTestModes[dwTestModes] := g_aDevices[dwDeviceIndex].aModeSize[i];
        inc( dwTestModes );
      end
    end;

    // Perform test on each of the selected modes on the selected device
    hr := PerformDirectDrawRefreshRate( pDD, aTestModes, dwTestModes );

    // Release this device
    pDD := nil;

    case hr of
      DDERR_NOMONITORINFORMATION:
      begin
        // No EDID data is present for the current monitor.
        MessageBox(g_hDlg,
            'The current monitor cannot be identified electronically.'  + #13#10 +
            'High refresh rates are not allowed on such monitors, so the test will not be performed.',
            'Testing Will Not Proceed', MB_OK or MB_ICONINFORMATION);
      end;

      DDERR_NODRIVERSUPPORT:
      begin
        // The driver cannot support refresh rate testing.
        MessageBox(g_hDlg,
            'This driver does not support the refresh rate feature of DirectDraw.  Test cannot be performed.',
            'Testing Cannot Proceed', MB_OK or MB_ICONINFORMATION );
      end;
      else
      begin
        if( hr = DD_OK ) then
        begin
          MessageBox( g_hDlg, 'Mode test completed', 'Result', MB_OK or MB_ICONINFORMATION );
          result := hr;
          exit;
        end
        else
        begin
          // A StartRefreshRate error occurred.
          MessageBox(g_hDlg,
          'StartRefreshRate returned an unexpected value when called with the DDSMT_ISTESTREQUIRED flag.',
          'StartRefreshRate Error', MB_OK or MB_ICONEXCLAMATION );
          result := hr;
          exit;
        end;
      end;
    end;

    // Update the mode list boxes based on the device selected
    hr := UpdateModesListBox( dwDeviceIndex );
    if( hr <> DD_OK ) then
    begin
      result := hr;
      exit;
    end;
  end
  else
  begin
    // There weren't any modes selected to test
    MessageBox( g_hDlg, 'Select one or more modes to test from the list box',
                'No modes selected', MB_OK or MB_ICONINFORMATION );
  end;

  result := DD_OK;
end;

//-----------------------------------------------------------------------------
// Name: ResetDeviceModes()
// Desc: If the user makes a mistake, and accidently says YES when a mode 
//       cannot be seen, or NO when a mode really can be seen (thus ending up 
//       with a lower refresh rate than possible) this allows the user to reset 
//       the test results and try again.
//-----------------------------------------------------------------------------
function ResetDeviceModes( dwDeviceIndex : DWORD  ) : HRESULT;
var
  pDD : IDIRECTDRAW7;
  hr : HRESULT;
  Dummy : Pointer;
begin
  // Create a DirectDraw device based using the selected device guid
  hr := DirectDrawCreateEx( @g_aDevices[dwDeviceIndex].FGuid, pDD, IID_IDirectDraw7, nil );

  if( hr = DD_OK ) then
  begin
    // Set the cooperative level to normal
    hr := pDD.SetCooperativeLevel( g_hDlg, DDSCL_NORMAL );
    if( hr = DD_OK ) then
    begin
      // Clear the previous mode tests
      //
      // We ignore the return code, since we would do nothing different on error returns.
      // Note that most applications would never need to call StartRefreshRate this way.
      // The reset functionality is intended to be used when a user accidentally accepted
      // a mode that didn't actually display correctly.
      Dummy := nil;
      pDD.StartModeTest( Dummy, 0, 0 );
    end;

    // Release this device
    pDD := nil;
  end;

  hr := UpdateModesListBox( dwDeviceIndex );

  result := hr;
end;

//-----------------------------------------------------------------------------
// Name: InfoDlgProc()
// Desc: The dialog window proc
//-----------------------------------------------------------------------------
function InfoDlgProc( hDlg : HWND; amessage : UINT; wParm : WPARAM; lParm : LPARAM ) : Boolean; stdcall;
var
  hWndDeviceList : HWND;
  dwDeviceIndex : DWORD;
  hr: HResult;
begin
  case (amessage) of
    WM_INITDIALOG:
    begin
      // Store HWND in global
      g_hDlg := hDlg;
      // Setup the first devices text
      OnInitDialog;
    end;

    WM_COMMAND:
    begin
      case LOWORD(wParm) of
        IDCANCEL:
        begin
          EndDialog( hDlg, IDCANCEL );
        end;

        IDC_TEST:
        begin
          hr := OnRefreshRate;
          if( hr <> DD_OK ) then
          begin
            MessageBox( g_hDlg, 'Error doing starting mode test.' + #1310 +
                        'Sample will now exit.', 'DirectDraw RefreshRate Example',
                        MB_OK or MB_ICONERROR );
            EndDialog( g_hDlg, IDABORT );
          end;
        end;

        IDC_RESET:
        begin
          // Get the currently selected DirectDraw device
          hWndDeviceList := GetDlgItem( hDlg, IDC_DDRAW_DEVICE_LIST );
          dwDeviceIndex := SendMessage( hWndDeviceList, LB_GETCURSEL, 0, 0 );

          // Reset the modes for it
          hr := ResetDeviceModes( dwDeviceIndex );
          if( hr <> DD_OK ) then
          begin
            MessageBox( g_hDlg, 'Error reset DirectDraw device.' + #13#10 +
                        'Sample will now exit.', 'DirectDraw RefreshRate Example',
                        MB_OK or MB_ICONERROR );
            EndDialog( g_hDlg, IDABORT );
          end;
        end;

        IDC_DDRAW_DEVICE_LIST:
        begin
          case HIWORD(wParm) of
            LBN_SELCHANGE:
            begin
              // Get the currently selected DirectDraw device
              dwDeviceIndex := SendMessage( HWND( lParm ), LB_GETCURSEL, 0, 0 );

              // Update the list boxes using it
              hr := UpdateModesListBox( dwDeviceIndex );
              if( hr <> DD_OK ) then
              begin
                MessageBox( g_hDlg, 'Error enumerating DirectDraw modes.' + #13#10 +
                            'Sample will now exit.', 'DirectDraw Sample',
                            MB_OK or MB_ICONERROR );
                EndDialog( g_hDlg, IDABORT );
              end;
            end;

            else
            begin
              result := FALSE; // Message not handled
              exit;
            end;
          end;
        end;

        else
        begin
          result := FALSE; // Message not handled
          exit;
        end;
      end;
    end;

    else
    begin
      result := FALSE; // Message not handled
      exit;
    end;
  end;

  result := TRUE; // Message handled
end;

//-----------------------------------------------------------------------------
// Name: EnumModesCallback()
// Desc: Enumerates the available modes for the device from which 
//       EnumDisplayModes() was called.  It records the unique mode sizes in 
//       the g_aDevices[g_dwDeviceCount].aModeSize array
//-----------------------------------------------------------------------------
function EnumModesCallback( const pddsd : TDDSURFACEDESC; pContext : Pointer ) : HResult; stdcall; //RZ Added Const
var
  i : DWORD;
  dwModeSizeX : DWORD;
  dwModeSizeY : DWORD;
  dwModeCount : DWORD ;
begin
  // For each mode, look through all previously found modes
  // to see if this mode has already been added to the list
  dwModeCount := g_aDevices[ g_dwDeviceCount ].dwModeCount;
  //really 0 means current display mode...
  i:= 0;                     //RZ Must Be 0 Before Loop
  if dwModeCount<>0 then     //RZ We Don't Have It
    for i := 0 to dwModeCount - 1 do
    begin
      dwModeSizeX := g_aDevices[ g_dwDeviceCount ].aModeSize[i].cx;
      dwModeSizeY := g_aDevices[ g_dwDeviceCount ].aModeSize[i].cy;

      if ( ( dwModeSizeX = pddsd.dwWidth ) and ( dwModeSizeY = pddsd.dwHeight ) ) then
      begin
        // If this mode has been added, then stop looking
        break;
      end;
    end;

  // If this mode was not in g_aDevices[g_dwDeviceCount].aModeSize[]
  // then added it.
  if( g_aDevices[ g_dwDeviceCount ].dwModeCount = i) then
  begin
    g_aDevices[ g_dwDeviceCount ].aModeSize[i].cx := pddsd.dwWidth;
    g_aDevices[ g_dwDeviceCount ].aModeSize[i].cy := pddsd.dwHeight;

    // Increase the number of modes found for this device
    inc( g_aDevices[ g_dwDeviceCount ].dwModeCount );
  end;

  result := DDENUMRET_OK;
end;

//-----------------------------------------------------------------------------
// Name: DDEnumCallbackEx()
// Desc: This callback gets the information for each device enumerated
//-----------------------------------------------------------------------------
function DDEnumCallbackEx( pGUID : PGUID; strDescription : PAnsiChar; strDriverName : PAnsiChar;  pContext : Pointer; hm : HMONITOR ) : LongBool; stdcall;
var
  pDD : IDIRECTDRAW7;
  hr : HRESULT;
begin
  pDD := nil;

  // Create a DirectDraw object using the enumerated GUID
  hr := DirectDrawCreateEx( pGUID, pDD, IID_IDirectDraw7, nil );
  if( hr <> DD_OK ) then
  begin
    result := False;
    exit;
  end
  else
  begin
    if pGUID <> nil then
    begin
      // Add it to the global storage structure
      g_aDevices[ g_dwDeviceCount ].FGuid := pGUID^;
    end
    else
    begin
      // Clear the guid from the global storage structure
      FillChar( g_aDevices[ g_dwDeviceCount ].FGuid, SizeOf(TGUID), 0 );
    end;

    // Copy the description of the driver into the structure
    g_aDevices[ g_dwDeviceCount ].strDescription := strDescription;
    g_aDevices[ g_dwDeviceCount ].strDriverName :=  strDriverName;

    // Retrive the modes this device can support
    g_aDevices[ g_dwDeviceCount ].dwModeCount := 0;
    hr := pDD.EnumDisplayModes( 0, nil, nil, @EnumModesCallback );
    if( hr <> DD_OK ) then
    begin
      result := False;
      exit;
    end;

    // Increase the counter for the number of devices found
    inc( g_dwDeviceCount );

    // Finished with the DirectDraw object, so release it
    pDD := nil;
  end;
  result := True;
end;

//-----------------------------------------------------------------------------
// Name: GetDirectDrawDevices()
// Desc: Retrieves all available DirectDraw devices and stores the information
//       in g_aDevices[]
//-----------------------------------------------------------------------------
function GetDirectDrawDevices : HRESULT;
begin
  result := DirectDrawEnumerateEx( @DDEnumCallbackEx,
                                   nil,
                                   DDENUM_ATTACHEDSECONDARYDEVICES or
                                   DDENUM_DETACHEDSECONDARYDEVICES or
                                   DDENUM_NONDISPLAYDEVICES );
end;

var
  hr : HRESULT;
begin
  hr := GetDirectDrawDevices;
  if hr <> DD_OK{DDENUMRET_OK} then //RZ  DirectDrawEnumerateEx Returns DD_OK
    exit;

  // Display the main dialog box.
  DialogBox( hInstance, MAKEINTRESOURCE(IDD_MAIN), GetDesktopWindow, @InfoDlgProc );
end.
