program SwitchScreenMode_api;
{******************************************************************}
{                                                                  }
{       Borland Delphi DirectX 8 SDK Examples                      }
{       Conversion of the Switch Screen Mode Example               }
{                                                                  }
{ Portions created by Microsoft are                                }
{ Copyright (C) 1995-2000 Microsoft Corporation.                   }
{ All Rights Reserved.                                             }
{                                                                  }
{ The original files are : SwitchScreenMode.h,                     }
{                          SwitchScreenMode.cpp                    }
{ The original Pascal code is : SwitchScreenMode_api.pas           }
{ The initial developer of the Pascal code is : Dominique Louis    }
{ ( Dominique@SavageSoftware.com.au )                              }
{                                                                  }
{ Portions created by Dominique Louis are                          }
{ Copyright (C) 2000 - 2001 Dominique Louis.                       }
{                                                                  }
{ Contributor(s)                                                   }
{ --------------                                                   }
{ <Contributer Name> ( contributer@sp.sw )                         }
{                                                                  }
{ Obtained through:                                                }
{ Joint Endeavour of Delphi Innovators ( Project JEDI )            }
{                                                                  }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{                                                                  }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/NPL/NPL-1_1Final.html                     }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{ Description                                                      }
{ -----------                                                      }
{   SwitchScreenMode demonstrates how to author a DirectDraw       }
{   application to handle switching between both full-screen and   }
{   windowed DirectDraw modes.                                     }
{                                                                  }
{ Requires                                                         }
{ --------                                                         }
{   Eric Unger and Tim Baumgarten's conversions of the DirectX     }
{   headers. They are available from...                            }
{   http://delphi-jedi.org/DelphiGraphics/ .                       }
{                                                                  }
{                                                                  }
{ Programming Notes                                                }
{ -----------------                                                }
{   For details on how to setup a full-screen DirectDraw app, see  }
{   the FullScreenMode sample.                                     }
{                                                                  }
{   For details on how to setup a windowed mode DirectDraw app, see}
{  the WindowedMode sample.                                        }
{                                                                  }
{   When combining these the two mode, be sure to re-create all    }
{   DirectDraw surfaces since the pixel format for full-screen     }
{   mode typically does not match the desktop pixel format.        }
{                                                                  }
{ Revision History                                                 }
{ ----------------                                                 }
{   2000-12-01 - DL  Initial translation.                          }
{                                                                  }
{                                                                  }
{******************************************************************}

uses
  Windows,
  Messages,
  DirectDraw,
  SysUtils,
  DDUtil in '..\..\Common\DDUtil.pas';

{$R *.res}

const
  NAME : PChar = 'SwitchScreenMode';
  TITLE : PChar = 'DirectDraw SwitchScreenMode Sample';
  IDI_MAIN_ICON = 101;
  IDR_MENU = 102;
  IDR_MAIN_ACCEL = 103;
  IDB_DIRECTX = 107;
  IDM_ABOUT = 1000;
  IDM_EXIT = 1001;
  IDM_TOGGLEFULLSCREEN = 1002;

  SCREEN_WIDTH = 640;
  SCREEN_HEIGHT = 480;
  SCREEN_BPP = 8;

  SPRITE_DIAMETER = 48;
  NUM_SPRITES = 25;

  WINDOWED_HELPTEXT = 'Press Escape to quit.  Press Alt-Enter to switch to Full-Screen mode.';
  FULLSCREEN_HELPTEXT = 'Press Escape to quit.  Press Alt-Enter to switch to Windowed mode.';

type
  PSpriteRecord = ^TSpriteRecord;
  TSpriteRecord = packed record
    fPosX : single;
    fPosY : single;
    fVelX : single;
    fVelY : single;
  end;

var
  g_pDisplay : TDisplay;
  g_pLogoSurface : TSurface;
  g_pTextSurface : TSurface;
  g_rcWindow : TRect; // Saves the window size & pos.
  g_bActive : Boolean = False; // Is application active?
  g_bWindowed : Boolean = True; // App is in windowed mode?
  g_dwLastTick : TDateTime;
  g_Sprite : array[ 0..NUM_SPRITES - 1 ] of TSpriteRecord;

{-----------------------------------------------------------------------------
  Name: FreeDirectDraw
  Desc: Release all the DirectDraw objects
-----------------------------------------------------------------------------}

procedure FreeDirectDraw;
begin
  g_pLogoSurface.Free;
  g_pLogoSurface := nil;
  g_pTextSurface.Free;
  g_pTextSurface := nil;
  g_pDisplay.Free;
  g_pDisplay := nil;
end;

{-----------------------------------------------------------------------------
   Name: UpdateSprite()
   Desc: Move the sprite around and make it bounce based on how much time
         has passed
-----------------------------------------------------------------------------}

procedure UpdateSprite( var pSprite : TSpriteRecord; fTimeDelta : Single );
begin
  // Update the sprite position
  pSprite.fPosX := pSprite.fPosX + pSprite.fVelX * fTimeDelta;
  pSprite.fPosY := pSprite.fPosY + pSprite.fVelY * fTimeDelta;

  // Clip the position, and bounce if it hits the edge
  // Clip the position, and bounce if it hits the edge
  if ( pSprite.fPosX < 0.0 ) then
  begin
    pSprite.fPosX := 0;
    pSprite.fVelX := -pSprite.fVelX;
  end;

  if ( pSprite.fPosX >= SCREEN_WIDTH - SPRITE_DIAMETER ) then
  begin
    pSprite.fPosX := SCREEN_WIDTH - 1 - SPRITE_DIAMETER;
    pSprite.fVelX := -pSprite.fVelX;
  end;

  if ( pSprite.fPosY < 0 ) then
  begin
    pSprite.fPosY := 0;
    pSprite.fVelY := -pSprite.fVelY;
  end;

  if ( pSprite.fPosY > SCREEN_HEIGHT - SPRITE_DIAMETER ) then
  begin
    pSprite.fPosY := SCREEN_HEIGHT - 1 - SPRITE_DIAMETER;
    pSprite.fVelY := -pSprite.fVelY;
  end;
end;

{-----------------------------------------------------------------------------
   Name: InitDirectDrawMode()
   Desc: Called when the user wants to toggle between full-screen and windowed
         to create all the needed DDraw surfaces and set the coop level
-----------------------------------------------------------------------------}

function InitDirectDrawMode( h_Wnd : HWND; bWindowed : Boolean ) : HRESULT;
var
  hr : HRESULT;
  pDDPal : IDIRECTDRAWPALETTE;
  iSprite : integer;
  dwStyle : DWORD;
  h_Inst : hInst;
  h_Menu : HMENU;
begin
  pDDpal := nil;

  // Release all existing surfaces
  FreeDirectDraw;

  // The back buffer and primary surfaces need to be created differently
  // depending on if we are in full-screen or windowed mode
  g_pDisplay := TDisplay.Create;

  if ( bWindowed ) then
  begin
    hr := g_pDisplay.CreateWindowedDisplay( h_Wnd, SCREEN_WIDTH, SCREEN_HEIGHT );
    if ( hr <> DD_OK ) then
    begin
      result := hr;
      exit;
    end;

    // Add the system menu to the window's style
    dwStyle := GetWindowLong( h_Wnd, GWL_STYLE );
    dwStyle := dwStyle or WS_SYSMENU;
    SetWindowLong( h_Wnd, GWL_STYLE, dwStyle );

      // Show the menu in windowed mode
{$IFDEF WIN64}
    h_Inst := GetWindowLongPtr( hWnd, GWLP_HINSTANCE );
{$ELSE}
    h_Inst := GetWindowLong( h_Wnd, GWL_HINSTANCE );
{$ENDIF}
    h_Menu := LoadMenu( h_Inst, MakeIntResource( IDR_MENU ) );
    SetMenu( h_Wnd, h_Menu );
  end
  else
  begin
    hr := g_pDisplay.CreateFullScreenDisplay( h_Wnd, SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP );
    if ( hr <> DD_OK ) then
    begin
      MessageBox( h_Wnd, 'This display card does not support 640x480x8.', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
      result := hr;
      exit
    end;

    // Disable the menu in full-screen since we are
    // using a palette and a menu would look bad
    SetMenu( h_Wnd, 0 );

    // Remove the system menu from the window's style
    dwStyle := GetWindowLong( h_Wnd, GWL_STYLE );
    dwStyle := dwStyle and not WS_SYSMENU;
    SetWindowLong( h_Wnd, GWL_STYLE, dwStyle );
  end;

  // We need to release and re-load, and set the palette again to
  // redraw the bitmap on the surface.  Otherwise, GDI will not
  // draw the bitmap on the surface with the right palette
  hr := g_pDisplay.CreatePaletteFromBitmap( pDDPal, PAnsiChar( '..\media\directx.bmp' ) {MakeIntResource( IDB_DIRECTX ) } );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit
  end;

  g_pDisplay.SetPalette( pDDPal );

  pDDPal := nil;

  if ( g_bWindowed ) then
  begin
    // Create a surface, and draw text to it.
    hr := g_pDisplay.CreateSurfaceFromText( g_pTextSurface, 0, WINDOWED_HELPTEXT, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
    if ( hr <> DD_OK ) then
    begin
      result := hr;
      exit;
    end;
  end
  else
  begin
    // Create a surface, and draw text to it.
    hr := g_pDisplay.CreateSurfaceFromText( g_pTextSurface, 0, FULLSCREEN_HELPTEXT, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
    if ( hr <> DD_OK ) then
    begin
      result := hr;
      exit;
    end;
  end;


  // Create a surface, and draw a bitmap resource on it.  The surface must
  // be newly created every time the screen mode is switched since it
  // uses the pixel format of the primary surface
  hr := g_pDisplay.CreateSurfaceFromBitmap( g_pLogoSurface, PAnsiChar( '..\media\directx.bmp' ) {MakeIntResource( IDB_DIRECTX )}, SPRITE_DIAMETER, SPRITE_DIAMETER );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  // Set the color key for the logo sprite to black
  hr := g_pLogoSurface.SetColorKey( 0 );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  // Init all the sprites.  All of these sprites look the same,
  // using the g_pDDSLogo surface.
  for iSprite := 0 to NUM_SPRITES - 1 do
  begin
    // Set the sprite's position and velocity
    g_Sprite[ iSprite ].fPosX := random( SCREEN_WIDTH );
    g_Sprite[ iSprite ].fPosY := random( SCREEN_HEIGHT );

    g_Sprite[ iSprite ].fVelX := 500.0 * random / RAND_MAX - 250.0;
    g_Sprite[ iSprite ].fVelY := 500.0 * random / RAND_MAX - 250.0;
  end;

  result := DD_OK;
end;


{-----------------------------------------------------------------------------
   Name: DisplayFrame()
   Desc: Blts a the sprites to the back buffer, then it blts or flips the
         back buffer onto the primary buffer.
-----------------------------------------------------------------------------}

function DisplayFrame : HRESULT;
var
  hr : HRESULT;
  iSprite : integer;
begin
  // Fill the back buffer with black, ignoring errors until the flip
  g_pDisplay.Clear( 0 );

  // Blt the help text on the backbuffer, ignoring errors until the flip
  g_pDisplay.Blt( 10, 10, g_pTextSurface, nil );

  // Blt all the sprites onto the back buffer using color keying,
  // ignoring errors until the flip. Note that all of these sprites
  // use the same DirectDraw surface.
  for iSprite := 0 to NUM_SPRITES - 1 do
    g_pDisplay.Blt( Round( g_Sprite[ iSprite ].fPosX ), Round( g_Sprite[ iSprite ].fPosY ), g_pLogoSurface, nil );


  // Flip or blt the back buffer onto the primary buffer
  hr := g_pDisplay.Flip;
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: RestoreSurfaces()
   Desc: Restore all the surfaces, and redraw the sprite surfaces.
-----------------------------------------------------------------------------}

function RestoreSurfaces : HRESULT;
var
  hr : HRESULT;
  pDDPal : IDIRECTDRAWPALETTE;
begin
  pDDPal := nil;

  hr := g_pDisplay.GetDirectDraw.RestoreAllSurfaces;
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  // No need to re-create the surface, just re-draw it.
  // We need to release and re-load, and set the palette again to
  hr := g_pDisplay.CreatePaletteFromBitmap( pDDPal, PAnsiChar( '..\media\directx.bmp' ) {MakeIntResource( IDB_DIRECTX ) } );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  g_pDisplay.SetPalette( pDDPal );

  pDDPal := nil;

  // No need to re-create the surface, just re-draw it.
  hr := g_pLogoSurface.DrawBitmap( PAnsiChar( '..\media\directx.bmp' ) {MakeIntResource( IDB_DIRECTX )}, SPRITE_DIAMETER, SPRITE_DIAMETER );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  if ( g_bWindowed ) then
  begin
    // No need to re-create the surface, just re-draw it.
    hr := g_pTextSurface.DrawText( 0, WINDOWED_HELPTEXT, 0, 0, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
    if ( hr <> DD_OK ) then
    begin
      result := hr;
      exit;
    end;
  end
  else
  begin
    // No need to re-create the surface, just re-draw it.
    hr := g_pTextSurface.DrawText( 0, FULLSCREEN_HELPTEXT, 0, 0, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
    if ( hr <> DD_OK ) then
    begin
      result := hr;
      exit;
    end;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: ProcessNextFrame()
   Desc: Move the sprites, blt them to the back buffer, then
         flip or blt the back buffer to the primary buffer
-----------------------------------------------------------------------------}

function ProcessNextFrame( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
  dwCurrTick : TDateTime;
  dwTickDiff : TDateTime;
  iSprite : integer;
begin
  // Figure how much time has passed since the last time
  dwCurrTick := GetTickCount;
  dwTickDiff := dwCurrTick - g_dwLastTick;

  // Don't update if no time has passed
  if ( dwTickDiff = 0 ) then
  begin
    result := DD_OK;
    exit
  end;

  g_dwLastTick := dwCurrTick;

  // Move the sprites according to how much time has passed
  for iSprite := 0 to NUM_SPRITES - 1 do
    UpdateSprite( g_Sprite[ iSprite ], dwTickDiff / 1000.0 );

  // Check the cooperative level before rendering
  hr := g_pDisplay.GetDirectDraw.TestCooperativeLevel;
  if ( hr <> DD_OK ) then
  begin
    case hr of

      DDERR_EXCLUSIVEMODEALREADYSET :
        begin
        // Do nothing because some other app has exclusive mode
          Sleep( 10 );
          result := DD_OK;
          exit;
        end;

      DDERR_NOEXCLUSIVEMODE :
        begin
        // Do nothing because some other app has exclusive mode
          Sleep( 10 );
          result := DD_OK;
          exit;
        end;

      DDERR_WRONGMODE :
        begin
        // The display mode changed on us. Update the
        // DirectDraw surfaces accordingly
          result := InitDirectDrawMode( h_Wnd, g_bWindowed );
          exit;
        end;
    end;
    result := hr;
    exit;
  end;

  // Display the sprites on the screen
  hr := DisplayFrame;
  if ( hr <> DD_OK ) then
  begin
    if ( hr <> DDERR_SURFACELOST ) then
    begin
      result := hr;
      exit;
    end;

    // The surfaces were lost so restore them
    RestoreSurfaces;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: WindowProc
   Desc: The Main Window Procedure
{-----------------------------------------------------------------------------}

function WindowProc( h_Wnd : THandle; aMSG : Cardinal; wParam : Cardinal; lParam : Integer ) : Integer; stdcall;
var
  pMinMax : TMinMaxInfo;
  dwFrameWidth, dwFrameHeight, dwMenuHeight, dwCaptionHeight : DWORD;
  pDDPal : IDIRECTDRAWPALETTE;
  hr : HRESULT;
begin
  case aMSG of
    // Pause if minimized
    WM_COMMAND :
      begin
        case LOWORD( wParam ) of
          IDM_TOGGLEFULLSCREEN :
            begin
          // Toggle the fullscreen/window mode
              if ( g_bWindowed ) then
                GetWindowRect( h_Wnd, g_rcWindow );

              g_bWindowed := not g_bWindowed;

              hr := InitDirectDrawMode( h_Wnd, g_bWindowed );
              if ( hr <> DD_OK ) then
              begin
                g_pDisplay := nil;

                MessageBox( h_Wnd, 'InitDirectDrawMode failed.' + #13#10 + 'The sample will now exit. ', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
                PostMessage( h_Wnd, WM_CLOSE, 0, 0 );
              end;
              result := 0;
              exit;
            end;

          IDM_EXIT :
            begin
          // Received key/menu command to exit app
              PostMessage( h_Wnd, WM_CLOSE, 0, 0 );
              result := 0;
              exit;
            end;
        end;
      end;


    WM_GETMINMAXINFO :
      begin
      // Don't allow resizing in windowed mode.
      // Fix the size of the window to 640x480 (client size)
        pMinMax := pMinMaxinfo( lParam )^;

        dwFrameWidth := GetSystemMetrics( SM_CXSIZEFRAME );
        dwFrameHeight := GetSystemMetrics( SM_CYSIZEFRAME );
        dwMenuHeight := GetSystemMetrics( SM_CYMENU );
        dwCaptionHeight := GetSystemMetrics( SM_CYCAPTION );

        pMinMax.ptMinTrackSize.x := SCREEN_WIDTH + dwFrameWidth * 2;
        pMinMax.ptMinTrackSize.y := SCREEN_HEIGHT + dwFrameHeight * 2 + dwMenuHeight + dwCaptionHeight;

        pMinMax.ptMaxTrackSize.x := pMinMax.ptMinTrackSize.x;
        pMinMax.ptMaxTrackSize.y := pMinMax.ptMinTrackSize.y;
        result := 0;
        exit;
      end;

    WM_MOVE :
      begin
      // Retrieve the window position after a move.
        if ( g_pDisplay <> nil ) then
          g_pDisplay.UpdateBounds;
        result := 0;
        exit;
      end;

    WM_SIZE :
      begin
      // Check to see if we are losing our window...
        if ( SIZE_MAXHIDE = wParam ) or ( SIZE_MINIMIZED = wParam ) then
          g_bActive := FALSE
        else
          g_bActive := TRUE;

        if ( g_pDisplay <> nil ) then
          g_pDisplay.UpdateBounds;
      end;

    WM_SETCURSOR :
      begin
      // Hide the cursor if in fullscreen
        if ( not g_bWindowed ) then
        begin
          SetCursor( 0 );
          result := 1;
          exit;
        end;
      end;

    WM_QUERYNEWPALETTE :
      begin
        if ( g_pDisplay <> nil ) and ( g_pDisplay.GetFrontBuffer <> nil ) then
        begin
        // If we are in windowed mode with a desktop resolution in 8 bit
        // color, then the palette we created during init has changed
        // since then.  So get the palette back from the primary
        // DirectDraw surface, and set it again so that DirectDraw
        // realises the palette, then release it again.
          pDDPal := nil;
          g_pDisplay.GetFrontBuffer.GetPalette( pDDPal );
          g_pDisplay.GetFrontBuffer.SetPalette( pDDPal );
          pDDPal := nil;
        end;
      end;

    WM_EXITMENULOOP :
      begin
      // Ignore time spent in menu
        g_dwLastTick := GetTickCount;
      end;

    WM_EXITSIZEMOVE :
      begin
      // Ignore time spent resizing
        g_dwLastTick := GetTickCount;
      end;

    WM_SYSCOMMAND :
      begin
      // Prevent moving/sizing and power loss in fullscreen mode
        case wParam of
          SC_MOVE, SC_SIZE, SC_MAXIMIZE, SC_MONITORPOWER :
            begin
              if ( not g_bWindowed ) then
              begin
                result := 1;
                exit;
              end;
            end;
        end;
      end;

    WM_DESTROY :
      begin
      // Cleanup and close the app
        FreeDirectDraw;
        PostQuitMessage( 0 );
        result := 0;
        exit;
      end;
  end;
  Result := DefWindowProc( h_Wnd, aMSG, wParam, lParam );
end;

{-----------------------------------------------------------------------------
   Name: WinInit
   Desc: Do work required for every instance of the application:
         Create the window, initialize data
-----------------------------------------------------------------------------}

function WinInit( h_Inst : THANDLE; nCmdShow : Integer; var phWnd : THandle; var phAccel : HACCEL ) : HRESULT;
var
  h_Wnd : THandle;
  wc : TWNDCLASS;
  h_Accel : HACCEL;
  dwStyle, dwFrameWidth, dwFrameHeight, dwMenuHeight, dwCaptionHeight, dwWindowWidth, dwWindowHeight : DWORD;
begin
  // Register the window class
  //wc.cbSize        := sizeof(wc);
  wc.lpszClassName := NAME;
  wc.lpfnWndProc := @WindowProc;
  wc.style := CS_VREDRAW or CS_HREDRAW;
  wc.hInstance := h_Inst;
  wc.hIcon := LoadIcon( h_Inst, MakeIntResource( IDI_MAIN_ICON ) );
  //wc.hIconSm       := LoadIcon( h_Inst, MakeIntResource(IDI_MAIN_ICON) );
  wc.hCursor := LoadCursor( 0, IDC_ARROW );
  wc.hbrBackground := ( COLOR_WINDOW + 1 );
  wc.lpszMenuName := MakeIntResource( IDR_MENU );
  wc.cbClsExtra := 0;
  wc.cbWndExtra := 0;

  if ( RegisterClass( wc ) = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  // Load keyboard accelerators
  h_Accel := LoadAccelerators( h_Inst, MakeIntResource( IDR_MAIN_ACCEL ) );

  // Calculate the proper size for the window given a client of 640x480
  dwFrameWidth := GetSystemMetrics( SM_CXSIZEFRAME );
  dwFrameHeight := GetSystemMetrics( SM_CYSIZEFRAME );
  dwMenuHeight := GetSystemMetrics( SM_CYMENU );
  dwCaptionHeight := GetSystemMetrics( SM_CYCAPTION );
  dwWindowWidth := SCREEN_WIDTH + dwFrameWidth * 2;
  dwWindowHeight := SCREEN_HEIGHT + dwFrameHeight * 2 + dwMenuHeight + dwCaptionHeight;

  // Create and show the main window
  dwStyle := WS_OVERLAPPEDWINDOW and not WS_MAXIMIZEBOX;
  h_Wnd := CreateWindowEx( 0,
    NAME,
    TITLE,
    dwStyle,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    dwWindowWidth,
    dwWindowHeight,
    0,
    0,
    h_Inst,
    nil );
  if ( h_Wnd = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  ShowWindow( h_Wnd, nCmdShow );
  UpdateWindow( h_Wnd );

  // Save the window size/pos for switching modes
  GetWindowRect( h_Wnd, g_rcWindow );

  phWnd := h_Wnd;
  phAccel := h_Accel;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: InitDirectDraw()
   Desc: Create the DirectDraw object, and init the surfaces
-----------------------------------------------------------------------------}

function InitDirectDraw( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
begin
  // Initialize all the surfaces we need
  hr := InitDirectDrawMode( h_Wnd, g_bWindowed );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;
  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: WinMain
   Desc: Entry point to the program. Initializes everything and calls
         UpdateFrame() when idle from the message pump.
-----------------------------------------------------------------------------}
var
  aMSG : MSG;
  h_Wnd : THandle;
  h_Accel : HACCEL;
  hr : HRESULT;
begin
  ZeroMemory( @g_Sprite, SizeOf( TSpriteRecord ) * NUM_SPRITES );

  random( GetTickCount );
  hr := WinInit( hInstance, SW_SHOW, h_Wnd, h_Accel );
  if ( hr <> DD_OK ) then
  begin
    exit;
  end;

  hr := InitDirectDraw( h_Wnd );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'DirectDraw init failed. ' + #13#10 + 'The sample will now exit. ', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
    exit;
  end;

  g_dwLastTick := GetTickCount;

  while ( true ) do
  begin
    // Look for messages, if none are found then
    // update the state and display it
    if ( PeekMessage( aMSG, 0, 0, 0, PM_NOREMOVE ) ) then
    begin
      if not GetMessage( aMSG, 0, 0, 0 ) then
        // WM_QUIT was posted, so exit
        exit;

      // Translate and dispatch the message
      if ( TranslateAccelerator( h_Wnd, h_Accel, aMsg ) = 0 ) then
      begin
        TranslateMessage( aMsg );
        DispatchMessage( aMsg );
      end;
    end
    else if ( g_bActive ) then
    begin
        // Move the sprites, blt them to the back buffer, then
        // flip or blt the back buffer to the primary buffer
      hr := ProcessNextFrame( h_Wnd );
      if ( hr <> DD_OK ) then
      begin
        g_pDisplay := nil;

        MessageBox( h_Wnd, 'Displaying the next frame failed.' + #13#10 + 'The sample will now exit.', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
        exit;
      end;
    end
    else
    begin
      // Go to sleep if we have nothing else to do
      WaitMessage;

      // Ignore time spent inactive
      g_dwLastTick := GetTickCount;
    end;
  end;
end.
