program WindowedMode_api;
{******************************************************************}
{                                                                  }
{       Borland Delphi DirectX 8 SDK Examples                      }
{       Conversion of the Windowed Mode Example                    }
{                                                                  }
{ Portions created by Microsoft are                                }
{ Copyright (C) 1995-2000 Microsoft Corporation.                   }
{ All Rights Reserved.                                             }
{                                                                  }
{ The original files are : WindowedMode.h,                         }
{                          WindowedMode.cpp                        }
{ The original Pascal code is : WindowedMode_api.pas               }
{ The initial developer of the Pascal code is : Dominique Louis    }
{ ( Dominique@SavageSoftware.com.au )                              }
{                                                                  }
{ Portions created by Dominique Louis are                          }
{ Copyright (C) 2000 - 2001 Dominique Louis.                       }
{                                                                  }
{ Contributor(s)                                                   }
{ --------------                                                   }
{ <Contributer Name> ( contributer@sp.sw )                         }
{                                                                  }
{ Obtained through:                                                }
{ Joint Endeavour of Delphi Innovators ( Project JEDI )            }
{                                                                  }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{                                                                  }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/NPL/NPL-1_1Final.html                     }
{                                                                  }
{ Software distributed under the License is distributed on an      }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License.                        }
{                                                                  }
{ Description                                                      }
{ -----------                                                      }
{   WindowedMode demonstrates the tasks required to initialize     }
{   and run a windowed DirectDraw application.                     }
{                                                                  }
{ Requires                                                         }
{ --------                                                         }
{   Eric Unger and Tim Baumgarten's conversions of the DirectX     }
{   headers. They are available from...                            }
{   http://delphi-jedi.org/DelphiGraphics/ .                       }
{                                                                  }
{                                                                  }
{ Programming Notes                                                }
{ -----------------                                                }
{   The basic tasks to author a simple windowed DirectDraw         }
{   application are as follows:                                    }
{                                                                  }
{   Initialize DirectDraw:                                         }
{     1. Register a window class and create a window.              }
{     2. Call DirectDrawCreateEx to create a DirectDraw object     }
{     3. Call SetCooperativeLevel to set the DirectDraw cooperative}
{        level to normal.                                          }
{     4. Call CreateSurface to obtain a pointer to the primary     }
{        surface.  In windowed mode, the primary surface is also   }
{        the desktop window.  So the pixel format for the primary  }
{        surface is based on the user's selection of display       }
{        resolution.                                               }
{     5. Create a back-buffer.  In windowed mode, this is should   }
{        be an off-screen plain buffer. The back buffer is         }
{        typically the same size as the window's client rect, but  }
{        could be created with any size.                           }
{     6. Test to see if the display mode is in palettized color    }
{        (8-bit or less).                                          }
{        If it is then a palette needs to be created. This sample  }
{        displays a single bitmap so it can read the bitmap        }
{        palette info to read and create a DirectDraw              }
{        palette.  After a palette is created, call SetPalette to  }
{        set the palette for the primary surface.                  }
{     7. Call CreateClipper and SetClipper to create a clipper     }
{        object, and attach it                                     }
{        to the primary surface.  This will keep DirectDraw from   }
{        blting on top of any windows which happen to overlap.     }
{     8. Create an off-screen plain DirectDraw surface, and load   }
{        media content into it. For example, this sample calls     }
{        DDUtil_CreateSurfaceFromBitmap() to do just this.         }
{                                                                  }
{   When the app is idle, and it is not hidden or minimized then   }
{   render the next frame by:                                      }
{     1. If movement or animation is involved in the app, then     }
{        calculate how much time has passed since the last time    }
{        the frame was displayed.                                  }
{     2. Move or animate the app state based on how much time has  }
{        passed.                                                   }
{     3. Draw the current state into the off-screen plain          }
{        backbuffer.                                               }
{     4. Call Blt to blt the contents of the off-screen plain      }
{        backbuffer into the primary surface.                      }
{                                                                  }
{   If the display resolution changes or an exclusive mode         }
{   DirectDraw app is run, then                                    }
{   the DirectDraw surface may be lost ( resulting in a DirectDraw }
{   call returning DDERR_SURFACELOST ), then handle it by:         }
{     1. Call RestoreAllSurfaces to have DirectDraw restore all    }
{        the surfaces.                                             }
{     2. Restoring a surface doesn't reload any content that       }
{        existed in the surface prior to it being lost. So you     }
{        must now redraw the graphics the surfaces once held. For  }
{        example, this sample handles this by calling              }
{        DDUtil_ReDrawBitmapOnDDS()                                }
{                                                                  }
{   In windowed mode, handle the following windows messages:       }
{     1. WM_PAINT: This is sent when all or a part of the window   }
{        is needs to be redrawn.  The app may not be active at the }
{        time this is called, so if this is not handled then the   }
{        window will appear blank.  To avoid this, make a call to  }
{        draw the next frame here.                                 }
{     2. WM_QUERYNEWPALETTE: This is sent when in a 8-bit desktop  }
{        and the another window set a new palette, but now this    }
{        window has control so it needs to reset its palette.  The }
{        easy way to make this happen in DirectDraw is just to     }
{        call SetPalette.  This will force DirectDraw to realize   }
{        the DirectDrawPalette attached to it.                     }
{     3. WM_MOVE: This is sent when the window is moving.  Record  }
{        the new window position here since the blt to the primary }
{        surface needs to know the window position.                }
{                                                                  }
{ Revision History                                                 }
{ ----------------                                                 }
{   2000-12-01 - DL  Initial translation.                          }
{                                                                  }
{                                                                  }
{******************************************************************}

uses
  Windows,
  Messages,
  SysUtils,
  DirectDraw,
  DDUtil in '..\..\Common\DDUtil.pas';

{$R *.res}

const
  NAME : PChar = 'WindowedMode';
  TITLE : PChar = 'DirectDraw Windowed Mode Example';
  IDI_MAIN_ICON = 101;
  IDR_MENU = 102;
  IDR_MAIN_ACCEL = 103;
  IDB_DIRECTX = 107;
  IDM_EXIT = 1001;

  SCREEN_WIDTH = 640;
  SCREEN_HEIGHT = 480;
  SCREEN_BPP = 8;

  SPRITE_DIAMETER = 48;
  NUM_SPRITES = 25;

  HELPTEXT = 'Press Escape to quit.';

type
  PSpriteRecord = ^TSpriteRecord;
  TSpriteRecord = packed record
    fPosX : single;
    fPosY : single;
    fVelX : single;
    fVelY : single;
  end;

var
  g_pDisplay : TDisplay;
  g_pLogoSurface : TSurface;
  g_pTextSurface : TSurface;
  g_bActive : Boolean = False; // Is application active?
  g_dwLastTick : DWord;
  g_Sprite : array[ 0..NUM_SPRITES - 1 ] of TSpriteRecord;

{-----------------------------------------------------------------------------
  Name: FreeDirectDraw
  Desc: Release all the DirectDraw objects
-----------------------------------------------------------------------------}

procedure FreeDirectDraw;
begin
  g_pLogoSurface.Free;
  g_pLogoSurface := nil;
  g_pTextSurface.Free;
  g_pTextSurface := nil;
  g_pDisplay.Free;
  g_pDisplay := nil;
end;

{-----------------------------------------------------------------------------
   Name: UpdateSprite()
   Desc: Move the sprite around and make it bounce based on how much time
         has passed
-----------------------------------------------------------------------------}

procedure UpdateSprite( var pSprite : TSpriteRecord; fTimeDelta : Single );
begin
  // Update the sprite position
  pSprite.fPosX := pSprite.fPosX + pSprite.fVelX * fTimeDelta;
  pSprite.fPosY := pSprite.fPosY + pSprite.fVelY * fTimeDelta;

  // Clip the position, and bounce if it hits the edge
  if ( pSprite.fPosX < 0.0 ) then
  begin
    pSprite.fPosX := 0;
    pSprite.fVelX := -pSprite.fVelX;
  end;

  if ( pSprite.fPosX >= SCREEN_WIDTH - SPRITE_DIAMETER ) then
  begin
    pSprite.fPosX := SCREEN_WIDTH - 1 - SPRITE_DIAMETER;
    pSprite.fVelX := -pSprite.fVelX;
  end;

  if ( pSprite.fPosY < 0 ) then
  begin
    pSprite.fPosY := 0;
    pSprite.fVelY := -pSprite.fVelY;
  end;

  if ( pSprite.fPosY > SCREEN_HEIGHT - SPRITE_DIAMETER ) then
  begin
    pSprite.fPosY := SCREEN_HEIGHT - 1 - SPRITE_DIAMETER;
    pSprite.fVelY := -pSprite.fVelY;
  end;
end;

{-----------------------------------------------------------------------------
   Name: InitDirectDraw()
   Desc: Create the DirectDraw object, and init the surfaces
-----------------------------------------------------------------------------}

function InitDirectDraw( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
  pDDPal : IDIRECTDRAWPALETTE;
  iSprite : integer;
begin
  pDDpal := nil;

  // The back buffer and primary surfaces need to be created differently
  // depending on if we are in full-screen or windowed mode
  g_pDisplay := TDisplay.Create;

  hr := g_pDisplay.CreateWindowedDisplay( h_Wnd, SCREEN_WIDTH, SCREEN_HEIGHT );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'Failed initializing DirectDraw.', TITLE, MB_ICONERROR or MB_OK );
    result := hr;
    exit;
  end;

  // Create and set the palette when in palettized color
  hr := g_pDisplay.CreatePaletteFromBitmap( pDDPal, PAnsiChar( '..\media\directx.bmp' ) {MakeIntResource( IDB_DIRECTX )} );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'Failed  CreatePaletteFromBitmap', TITLE, MB_ICONERROR or MB_OK );
    result := hr;
    exit
  end;

  g_pDisplay.SetPalette( pDDPal );

  pDDPal := nil;

  // Create a surface, and draw text to it.
  hr := g_pDisplay.CreateSurfaceFromText( g_pTextSurface, 0, HELPTEXT, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'Failed  CreateSurfaceFromText', TITLE, MB_ICONERROR or MB_OK );
    result := hr;
    exit;
  end;

  // Create a surface, and draw a bitmap resource on it.
  hr := g_pDisplay.CreateSurfaceFromBitmap( g_pLogoSurface, PAnsiChar( '..\media\directx.bmp' ) {MakeIntResource( IDB_DIRECTX )}, SPRITE_DIAMETER, SPRITE_DIAMETER );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'Failed  CreateSurfaceFromBitmap', TITLE, MB_ICONERROR or MB_OK );
    result := hr;
    exit;
  end;

  // Set the color key for the logo sprite to black
  hr := g_pLogoSurface.SetColorKey( 0 );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'Failed  SetColorKey', TITLE, MB_ICONERROR or MB_OK );
    result := hr;
    exit;
  end;

  // Init all the sprites.  All of these sprites look the same,
  // using the g_pDDSLogo surface.
  for iSprite := 0 to NUM_SPRITES - 1 do
  begin
    // Set the sprite's position and velocity
    g_Sprite[ iSprite ].fPosX := random( SCREEN_WIDTH );
    g_Sprite[ iSprite ].fPosY := random( SCREEN_HEIGHT );

    g_Sprite[ iSprite ].fVelX := 500.0 * random / RAND_MAX - 250.0;
    g_Sprite[ iSprite ].fVelY := 500.0 * random / RAND_MAX - 250.0;
  end;

  result := DD_OK;
end;


{-----------------------------------------------------------------------------
   Name: DisplayFrame()
   Desc: Blts a the sprites to the back buffer, then it blts or flips the
         back buffer onto the primary buffer.
-----------------------------------------------------------------------------}

function DisplayFrame : HRESULT;
var
  hr : HRESULT;
  iSprite : integer;
begin
  // Fill the back buffer with black, ignoring errors until the flip
  g_pDisplay.Clear( 0 );

  // Blt the help text on the backbuffer, ignoring errors until the flip
  g_pDisplay.Blt( 10, 10, g_pTextSurface, nil );

  // Blt all the sprites onto the back buffer using color keying,
  // ignoring errors until the flip. Note that all of these sprites
  // use the same DirectDraw surface.
  for iSprite := 0 to NUM_SPRITES - 1 do
    g_pDisplay.Blt( Round( g_Sprite[ iSprite ].fPosX ), Round( g_Sprite[ iSprite ].fPosY ), g_pLogoSurface, nil );


  // Flip or blt the back buffer onto the primary buffer
  hr := g_pDisplay.Flip;
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: RestoreSurfaces()
   Desc: Restore all the surfaces, and redraw the sprite surfaces.
-----------------------------------------------------------------------------}

function RestoreSurfaces : HRESULT;
var
  hr : HRESULT;
  pDDPal : IDIRECTDRAWPALETTE;
begin
  pDDPal := nil;

  hr := g_pDisplay.GetDirectDraw.RestoreAllSurfaces;
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  // No need to re-create the surface, just re-draw it.
  // We need to release and re-load, and set the palette again
  hr := g_pDisplay.CreatePaletteFromBitmap( pDDPal, PAnsiChar( '..\media\directx.bmp' ) {MakeIntResource( IDB_DIRECTX ) } );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  g_pDisplay.SetPalette( pDDPal );

  pDDPal := nil;

  // No need to re-create the surface, just re-draw it.
  hr := g_pLogoSurface.DrawBitmap( PAnsiChar( '..\media\directx.bmp' ) {MakeIntResource( IDB_DIRECTX )}, SPRITE_DIAMETER, SPRITE_DIAMETER );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  // No need to re-create the surface, just re-draw it.
  hr := g_pTextSurface.DrawText( 0, HELPTEXT, 0, 0, RGB( 0, 0, 0 ), RGB( 255, 255, 0 ) );
  if ( hr <> DD_OK ) then
  begin
    result := hr;
    exit;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: ProcessNextFrame()
   Desc: Move the sprites, blt them to the back buffer, then
         flip or blt the back buffer to the primary buffer
-----------------------------------------------------------------------------}

function ProcessNextFrame( h_Wnd : HWND ) : HRESULT;
var
  hr : HRESULT;
  dwCurrTick : DWord;
  dwTickDiff : DWord;
  iSprite : integer;
begin
    // Figure how much time has passed since the last time
  dwCurrTick := GetTickCount;
  dwTickDiff := dwCurrTick - g_dwLastTick;

    // Don't update if no time has passed
  if ( dwTickDiff = 0 ) then
  begin
    result := DD_OK;
    exit
  end;

  g_dwLastTick := dwCurrTick;

    // Move the sprites according to how much time has passed
  for iSprite := 0 to NUM_SPRITES - 1 do
    UpdateSprite( g_Sprite[ iSprite ], dwTickDiff / 1000.0 );

    // Check the cooperative level before rendering
  hr := g_pDisplay.GetDirectDraw.TestCooperativeLevel;
  if ( hr <> DD_OK ) then
  begin
    case hr of

      DDERR_EXCLUSIVEMODEALREADYSET :
        begin
          // Do nothing because some other app has exclusive mode
          Sleep( 10 );
          result := DD_OK;
          exit;
        end;

      DDERR_WRONGMODE :
        begin
          // The display mode changed on us. Update the
          // DirectDraw surfaces accordingly
          result := InitDirectDraw( h_Wnd );
          exit;
        end;
    end;
    result := hr;
    exit;
  end;

    // Display the sprites on the screen
  hr := DisplayFrame;
  if ( hr <> DD_OK ) then
  begin
    if ( hr <> DDERR_SURFACELOST ) then
    begin
      result := hr;
      exit;
    end;

      // The surfaces were lost so restore them
    RestoreSurfaces;
  end;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: WindowProc
   Desc: The Main Window Procedure
{-----------------------------------------------------------------------------}

function WindowProc( h_Wnd : THandle; aMSG : Cardinal; wParam : Cardinal; lParam : Integer ) : Integer; stdcall;
var
  pMinMax : TMinMaxInfo;
  dwFrameWidth, dwFrameHeight, dwMenuHeight, dwCaptionHeight : DWORD;
  pDDPal : IDIRECTDRAWPALETTE;
begin
  case aMSG of
    // Pause if minimized
    WM_COMMAND :
      begin
        case LOWORD( wParam ) of
          IDM_EXIT :
            begin
          // Received key/menu command to exit app
              PostMessage( h_Wnd, WM_CLOSE, 0, 0 );
              result := 0;
              exit;
            end;
        end;
      end;

    WM_PAINT :
      begin
      // Update the screen if we need to refresh. This case occurs
      // when in windowed mode and the window is behind others.
      // The app will not be active, but it will be visible.
        if ( g_pDisplay <> nil ) then
        begin
        // Display the new position of the sprite
          if ( DisplayFrame = DDERR_SURFACELOST ) then
          begin
          // If the surfaces were lost, then restore and try again
            RestoreSurfaces;
            DisplayFrame;
          end;
        end;
      end;

    WM_QUERYNEWPALETTE :
      begin
        if ( g_pDisplay <> nil ) and ( g_pDisplay.GetFrontBuffer <> nil ) then
        begin
        // If we are in windowed mode with a desktop resolution in 8 bit
        // color, then the palette we created during init has changed
        // since then.  So get the palette back from the primary
        // DirectDraw surface, and set it again so that DirectDraw
        // realises the palette, then release it again.
          pDDPal := nil;
          g_pDisplay.GetFrontBuffer.GetPalette( pDDPal );
          g_pDisplay.GetFrontBuffer.SetPalette( pDDPal );
          pDDPal := nil;
        end;
      end;

    WM_GETMINMAXINFO :
      begin
      // Don't allow resizing in windowed mode.
      // Fix the size of the window to 640x480 (client size)
        pMinMax := pMinMaxinfo( lParam )^;

        dwFrameWidth := GetSystemMetrics( SM_CXSIZEFRAME );
        dwFrameHeight := GetSystemMetrics( SM_CYSIZEFRAME );
        dwMenuHeight := GetSystemMetrics( SM_CYMENU );
        dwCaptionHeight := GetSystemMetrics( SM_CYCAPTION );

        pMinMax.ptMinTrackSize.x := SCREEN_WIDTH + dwFrameWidth * 2;
        pMinMax.ptMinTrackSize.y := SCREEN_HEIGHT + dwFrameHeight * 2 + dwMenuHeight + dwCaptionHeight;

        pMinMax.ptMaxTrackSize.x := pMinMax.ptMinTrackSize.x;
        pMinMax.ptMaxTrackSize.y := pMinMax.ptMinTrackSize.y;
        result := 0;
        exit;
      end;

    WM_MOVE :
      begin
      // Retrieve the window position after a move.
        if ( g_pDisplay <> nil ) then
          g_pDisplay.UpdateBounds;
        result := 0;
        exit;
      end;

    WM_SIZE :
      begin
      // Check to see if we are losing our window...
        if ( wParam = SIZE_MAXHIDE ) or ( wParam = SIZE_MINIMIZED ) then
          g_bActive := False
        else
          g_bActive := True;

        if ( g_pDisplay <> nil ) then
          g_pDisplay.UpdateBounds;
      end;

    WM_EXITMENULOOP :
      begin
      // Ignore time spent in menu
        g_dwLastTick := GetTickCount;
      end;

    WM_EXITSIZEMOVE :
      begin
      // Ignore time spent resizing
        g_dwLastTick := GetTickCount;
      end;

    WM_DESTROY :
      begin
      // Cleanup and close the app
        FreeDirectDraw;
        PostQuitMessage( 0 );
        result := 0;
        exit;
      end;
  end;
  Result := DefWindowProc( h_Wnd, aMSG, wParam, lParam );
end;

{-----------------------------------------------------------------------------
   Name: WinInit
   Desc: Do work required for every instance of the application:
         Create the window, initialize data
-----------------------------------------------------------------------------}

function WinInit( h_Inst : THANDLE; nCmdShow : Integer; var phWnd : THandle; var phAccel : HACCEL ) : HRESULT;
var
  h_Wnd : THandle;
  wc : TWndClassEx;
  h_Accel : HACCEL;
  dwStyle, dwFrameWidth, dwFrameHeight, dwMenuHeight, dwCaptionHeight, dwWindowWidth, dwWindowHeight : DWORD;
begin
  // Register the window class
  wc.cbSize := SizeOf( wc );
  wc.lpszClassName := NAME;
  wc.lpfnWndProc := @WindowProc;
  wc.style := CS_VREDRAW or CS_HREDRAW;
  wc.hInstance := h_Inst;
  wc.hIcon := LoadIcon( h_Inst, MakeIntResource( IDI_MAIN_ICON ) );
  wc.hIconSm := LoadIcon( h_Inst, MakeIntResource( IDI_MAIN_ICON ) );
  wc.hCursor := LoadCursor( 0, IDC_ARROW );
  wc.hbrBackground := ( COLOR_WINDOW + 1 );
  wc.lpszMenuName := MakeIntResource( IDR_MENU );
  wc.cbClsExtra := 0;
  wc.cbWndExtra := 0;

  if ( RegisterClassEx( wc ) = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  // Load keyboard accelerators
  h_Accel := LoadAccelerators( h_Inst, MakeIntResource( IDR_MAIN_ACCEL ) );

  // Calculate the proper size for the window given a client of 640x480
  dwFrameWidth := GetSystemMetrics( SM_CXSIZEFRAME );
  dwFrameHeight := GetSystemMetrics( SM_CYSIZEFRAME );
  dwMenuHeight := GetSystemMetrics( SM_CYMENU );
  dwCaptionHeight := GetSystemMetrics( SM_CYCAPTION );
  dwWindowWidth := SCREEN_WIDTH + dwFrameWidth * 2;
  dwWindowHeight := SCREEN_HEIGHT + dwFrameHeight * 2 + dwMenuHeight + dwCaptionHeight;

  // Create and show the main window
  dwStyle := WS_OVERLAPPEDWINDOW and ( not WS_MAXIMIZEBOX );
  h_Wnd := CreateWindowEx( 0,
    NAME,
    TITLE,
    dwStyle,
    CW_USEDEFAULT,
    CW_USEDEFAULT,
    dwWindowWidth,
    dwWindowHeight,
    0,
    0,
    h_Inst,
    nil );
  if ( h_Wnd = 0 ) then
  begin
    result := E_FAIL;
    exit;
  end;

  ShowWindow( h_Wnd, nCmdShow );
  UpdateWindow( h_Wnd );

  phWnd := h_Wnd;
  phAccel := h_Accel;

  result := DD_OK;
end;

{-----------------------------------------------------------------------------
   Name: WinMain
   Desc: Entry point to the program. Initializes everything and calls
         UpdateFrame() when idle from the message pump.
-----------------------------------------------------------------------------}
var
  aMSG : MSG;
  h_Wnd : THandle;
  h_Accel : HACCEL;
  hr : HRESULT;
begin
  ZeroMemory( @g_Sprite, SizeOf( TSpriteRecord ) * NUM_SPRITES );

  random( GetTickCount );
  hr := WinInit( hInstance, SW_SHOW, h_Wnd, h_Accel );
  if ( hr <> DD_OK ) then
  begin
    exit;
  end;

  hr := InitDirectDraw( h_Wnd );
  if ( hr <> DD_OK ) then
  begin
    MessageBox( h_Wnd, 'DirectDraw init failed. ' + #13#10 + 'The sample will now exit. ', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
    exit;
  end;

  g_dwLastTick := GetTickCount;

  while ( true ) do
  begin
    // Look for messages, if none are found then
    // update the state and display it
    if ( PeekMessage( aMSG, 0, 0, 0, PM_NOREMOVE ) ) then
    begin
      if not GetMessage( aMSG, 0, 0, 0 ) then
        // WM_QUIT was posted, so exit
        exit;

      // Translate and dispatch the message
      if ( TranslateAccelerator( h_Wnd, h_Accel, aMsg ) = 0 ) then
      begin
        TranslateMessage( aMsg );
        DispatchMessage( aMsg );
      end;
    end
    else if ( g_bActive ) then
    begin
        // Move the sprites, blt them to the back buffer, then
        // flip or blt the back buffer to the primary buffer
      hr := ProcessNextFrame( h_Wnd );
      if ( hr <> DD_OK ) then
      begin
        g_pDisplay := nil;

        MessageBox( h_Wnd, 'Displaying the next frame failed.' + #13#10 + 'The sample will now exit.', 'DirectDraw Sample', MB_ICONERROR or MB_OK );
        exit;
      end;
    end
    else
    begin
      // Go to sleep if we have nothing else to do
      WaitMessage;

      // Ignore time spent inactive
      g_dwLastTick := GetTickCount;
    end;
  end;
end.
